--
--	Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--	  as an unpublished work.
--

USE WORK.cypress.ALL;

PACKAGE memorypkg IS

    COMPONENT dsrlch 
	PORT (
	    D	: IN BIT;
	    S	: IN BIT;
	    R	: IN BIT;
	    E	: IN BIT;
	    Q	: OUT BIT
	);
    END COMPONENT;

	COMPONENT xdsrff 
	PORT (
	      X1 : IN BIT;
	      X2 : IN BIT;
	      S : IN BIT;
	      R : IN BIT;
	      CLK : IN BIT;
	      Q : OUT BIT
	);
        END COMPONENT;

	COMPONENT tsrff 
	PORT (
	      T : IN BIT;
	      S : IN BIT;
	      R : IN BIT;
	      CLK : IN BIT;
	      Q : OUT BIT
	);

        END COMPONENT;

	COMPONENT jksrff 
	PORT (
	      J : IN BIT;
	      K : IN BIT;
	      S : IN BIT;
	      R : IN BIT;
	      CLK : IN BIT;
	      Q : OUT BIT
	);
        END COMPONENT;

END memorypkg;

package memorypkg_local is
component dsrff
    port(
	d	: in bit;		-- D input to register.
	s	: in bit;		-- Set input Async.
	r	: in bit;		-- Reset input Async.
	clk	: in bit;		-- Positive edge Clock.
	q	: out bit		-- Output from register.
    );
end component;
end memorypkg_local ;

ENTITY dsrlch IS
    PORT (
	D	: IN BIT;
	S	: IN BIT;
	R	: IN BIT;
	E	: IN BIT;
	Q	: OUT BIT
    );
END dsrlch;

ENTITY xdsrff IS

	PORT (
	      X1 : IN BIT;
	      X2 : IN BIT;
	      S : IN BIT;
	      R : IN BIT;
	      CLK : IN BIT;
	      Q : OUT BIT
	);

END xdsrff;

ENTITY tsrff IS

	PORT (
	      T : IN BIT;
	      S : IN BIT;
	      R : IN BIT;
	      CLK : IN BIT;
	      Q : OUT BIT
	);

END tsrff;

ENTITY jksrff IS

	PORT (
	      J : IN BIT;
	      K : IN BIT;
	      S : IN BIT;
	      R : IN BIT;
	      CLK : IN BIT;
	      Q : OUT BIT
	);

END jksrff;

ARCHITECTURE archdsrlch of dsrlch IS
    signal qi: bit;
BEGIN
    qi	<= (NOT(R) AND (S OR (D AND E) OR (qi AND NOT(E))));
    Q	<= qi;
end archdsrlch;

USE WORK.memorypkg_local.all;

ARCHITECTURE archXDSRFF of xdsrff IS
	signal dtmp:bit;
BEGIN
	dtmp <= x1 xor x2;
	x:dsrff port map(dtmp,s,r,clk,q);
END archXDSRFF;

USE WORK.memorypkg_local.all;

ARCHITECTURE archTSRFF of tsrff IS
   signal dtmp,qtmp:bit;
BEGIN
	dtmp <= qtmp xor t;
	x:dsrff port map(dtmp,s,r,clk,qtmp);
	q <= qtmp;
END archTSRFF;

USE WORK.memorypkg_local.all;

ARCHITECTURE archJKSRFF of jksrff IS
	signal tmp,qtmp:bit;
BEGIN
	tmp <= ((J AND NOT qtmp) OR (NOT K AND QTMP));
	x:dsrff port map(tmp,s,r,clk,qtmp);
	q <= qtmp;
END archJKSRFF;
