--
--	Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--	  as an unpublished work.
--

PACKAGE shifterpkg IS

COMPONENT shift
    PORT (SI	: IN BIT;
	    L	: IN BIT;
	    D	: IN BIT;
	    EN	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q	: OUT BIT);
END COMPONENT;

COMPONENT unsr
    PORT (SL	: IN BIT;
	    SR	: IN BIT;
	    S0	: IN BIT;
	    S1	: IN BIT;
	    D	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q	: OUT BIT);
END COMPONENT;

COMPONENT shift4
    PORT (SI	: IN BIT;
	    L	: IN BIT;
	    D3, D2, D1, D0 	: IN BIT;
	    EN	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q3, Q2, Q1, Q0 	: OUT BIT);
END COMPONENT;

COMPONENT shift8
    PORT (SI	: IN BIT;
	    L	: IN BIT;
	    D7,D6,D5,D4,D3,D2,D1,D0 	: IN BIT;
	    EN	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q7,Q6,Q5,Q4,Q3,Q2,Q1,Q0 	: OUT BIT);
END COMPONENT;

COMPONENT shift16
    PORT (SI	: IN BIT;
	    L	: IN BIT;
	    D15,D14,D13,D12,D11,D10,D9,D8 	: IN BIT;
	    D7,D6,D5,D4,D3,D2,D1,D0 		: IN BIT;
	    EN	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q15,Q14,Q13,Q12,Q11,Q10,Q9,Q8 	: OUT BIT;
	    Q7,Q6,Q5,Q4,Q3,Q2,Q1,Q0 		: OUT BIT);
END COMPONENT;

COMPONENT unsr4
    PORT (SL	: IN BIT;
	    SR	: IN BIT;
	    S0	: IN BIT;
	    S1	: IN BIT;
	    D3, D2, D1, D0 	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q3, Q2, Q1, Q0 	: OUT BIT);
END COMPONENT;

COMPONENT unsr8
    PORT (SL	: IN BIT;
	    SR	: IN BIT;
	    S0	: IN BIT;
	    S1	: IN BIT;
	    D7,D6,D5,D4,D3,D2,D1,D0 	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q7,Q6,Q5,Q4,Q3,Q2,Q1,Q0 	: OUT BIT);
END COMPONENT;

COMPONENT unsr16
    PORT (SL	: IN BIT;
	    SR	: IN BIT;
	    S0	: IN BIT;
	    S1	: IN BIT;
	    D15,D14,D13,D12,D11,D10,D9,D8 	: IN BIT;
	    D7,D6,D5,D4,D3,D2,D1,D0 		: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q15,Q14,Q13,Q12,Q11,Q10,Q9,Q8 	: OUT BIT;
	    Q7,Q6,Q5,Q4,Q3,Q2,Q1,Q0 		: OUT BIT);
END COMPONENT;

END shifterpkg;

package shifterpkg_local is
component dsrff
    port(
	d	: in bit;		-- D input to register.
	s	: in bit;		-- Set input Async.
	r	: in bit;		-- Reset input Async.
	clk	: in bit;		-- Positive edge Clock.
	q	: out bit		-- Output from register.
    );
end component;
end shifterpkg_local ;

--
--	SHIFT register
--

ENTITY shift IS
    PORT( SI	: IN BIT;
	    L	: IN BIT;
	    D	: IN BIT;
	    EN	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q	: OUT BIT);
END shift;

USE WORK.shifterpkg_local.all;

ARCHITECTURE archSHIFT of shift IS

    signal dtmp, qtmp, s : bit;

begin

    dtmp <= d when (l = '1') else
	    si when (en = '1') else
	    qtmp;

    s <= '0';

    x:dsrff port map(dtmp,s,r,clk,qtmp);

    q <= qtmp;

END archSHIFT;

--
--	UNiversal Shift Register
--

ENTITY unsr IS
    PORT( SL	: IN BIT;
	    SR	: IN BIT;
	    S0	: IN BIT;
	    S1	: IN BIT;
	    D	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q	: OUT BIT);
END unsr;

USE WORK.shifterpkg_local.all;

ARCHITECTURE archUNSR of unsr IS

    signal dtmp, qtmp, s : bit;

begin

    dtmp <=  d when (s1 = '1' AND s0 = '1') else
	    sl when (s1 = '1' AND s0 = '0') else
	    sr when (s1 = '0' AND s0 = '1') else
	    qtmp; --(s1 = '0' AND s0 = '0')

    s <= '0';

    x:dsrff port map(dtmp,s,r,clk,qtmp);

    q <= qtmp;

END archUNSR;

--
--	SHIFT Register 4-bits
--

ENTITY shift4 IS
    PORT( SI	: IN BIT;
	    L	: IN BIT;
	    D3, D2, D1, D0 	: IN BIT;
	    EN	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q3, Q2, Q1, Q0 	: OUT BIT);
END shift4;

ARCHITECTURE archSHIFT4 of shift4 IS

    signal qtmp0, qtmp1, qtmp2, qtmp3 : bit;

BEGIN

    s1:shift port map(   si,l,d0,en,clk,r,qtmp0);
    s2:shift port map(qtmp0,l,d1,en,clk,r,qtmp1);
    s3:shift port map(qtmp1,l,d2,en,clk,r,qtmp2);
    s4:shift port map(qtmp2,l,d3,en,clk,r,qtmp3);

    q0 <= qtmp0;
    q1 <= qtmp1;
    q2 <= qtmp2;
    q3 <= qtmp3;

END archSHIFT4;

--
--	SHIFT Register 8-bits
--

ENTITY shift8 is
    PORT (SI	: IN BIT;
	    L	: IN BIT;
	    D7,D6,D5,D4,D3,D2,D1,D0 	: IN BIT;
	    EN	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q7,Q6,Q5,Q4,Q3,Q2,Q1,Q0 	: OUT BIT);
END shift8;

ARCHITECTURE archSHIFT8 of shift8 IS

    signal q3tmp : bit;

BEGIN

    U1: shift4 port map (si,l,d3,d2,d1,d0,en,clk,r,q3tmp,q2,q1,q0);
    U2: shift4 port map (q3tmp,l,d7,d6,d5,d4,en,clk,r,q7,q6,q5,q4);

	q3 <= q3tmp;

END archSHIFT8;

--
--	SHIFT Register 16-bits
--

ENTITY shift16 is
    PORT (SI	: IN BIT;
	    L	: IN BIT;
	    D15,D14,D13,D12,D11,D10,D9,D8 	: IN BIT;
	    D7,D6,D5,D4,D3,D2,D1,D0 		: IN BIT;
	    EN	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q15,Q14,Q13,Q12,Q11,Q10,Q9,Q8 	: OUT BIT;
	    Q7,Q6,Q5,Q4,Q3,Q2,Q1,Q0 		: OUT BIT);
END shift16;

ARCHITECTURE archSHIFT16 of shift16 IS

    signal q3tmp,q7tmp,qbtmp : bit;

BEGIN

    U1: shift4 port map (si,l,d3,d2,d1,d0,en,clk,r,q3tmp,q2,q1,q0);
    U2: shift4 port map (q3tmp,l,d7,d6,d5,d4,en,clk,r,q7tmp,q6,q5,q4);
    U3: shift4 port map (q7tmp,l,d11,d10,d9,d8,en,clk,r,qbtmp,q10,q9,q8);
    U4: shift4 port map (qbtmp,l,d15,d14,d13,d12,en,clk,r,q15,q14,q13,q12);

    q3  <= q3tmp;
    q7  <= q7tmp;
    q11 <= qbtmp;

END archSHIFT16;

--
--	UNiversal Shift Register 4-bits
--

ENTITY unsr4 IS
    PORT( SL	: IN BIT;
	    SR	: IN BIT;
	    S0	: IN BIT;
	    S1	: IN BIT;
	    D3, D2, D1, D0 	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q3, Q2, Q1, Q0 	: OUT BIT);
END unsr4;

ARCHITECTURE archUNSR4 of unsr4 IS

    signal qtmp0, qtmp1, qtmp2, qtmp3 : bit;

BEGIN

    u1:unsr port map(sl,   qtmp1,s0,s1,d0,clk,r,qtmp0);
    u2:unsr port map(qtmp0,qtmp2,s0,s1,d1,clk,r,qtmp1);
    u3:unsr port map(qtmp1,qtmp3,s0,s1,d2,clk,r,qtmp2);
    u4:unsr port map(qtmp2,sr   ,s0,s1,d3,clk,r,qtmp3);
    q0 <= qtmp0;
    q1 <= qtmp1;
    q2 <= qtmp2;
    q3 <= qtmp3;

END archUNSR4;

--
--	Universal Shift Register 8-bits
--

ENTITY unsr8 is
    PORT( SL	: IN BIT;
	    SR	: IN BIT;
	    S0	: IN BIT;
	    S1	: IN BIT;
	    D7,D6,D5,D4,D3,D2,D1,D0 	: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q7,Q6,Q5,Q4,Q3,Q2,Q1,Q0 	: OUT BIT);
END unsr8;

ARCHITECTURE archunsr8 of unsr8 IS

	signal q3tmp,q4tmp : bit;

BEGIN

    U1: unsr4 port map (   sl,q4tmp,s0,s1,d3,d2,d1,d0,clk,r,q3tmp,q2,q1,   q0);
    U2: unsr4 port map (q3tmp,   sr,s0,s1,d7,d6,d5,d4,clk,r,   q7,q6,q5,q4tmp);

    q3 <= q3tmp;
    q4 <= q4tmp;

END archunsr8;

--
--	Universal Shift Register 16-bits
--

ENTITY unsr16 is
    PORT( SL	: IN BIT;
	    SR	: IN BIT;
	    S0	: IN BIT;
	    S1	: IN BIT;
	    D15,D14,D13,D12,D11,D10,D9,D8 	: IN BIT;
	    D7,D6,D5,D4,D3,D2,D1,D0 		: IN BIT;
	    CLK	: IN BIT;
	    R	: IN BIT;
	    Q15,Q14,Q13,Q12,Q11,Q10,Q9,Q8 	: OUT BIT;
	    Q7,Q6,Q5,Q4,Q3,Q2,Q1,Q0 		: OUT BIT);
END unsr16;

ARCHITECTURE archunsr16 of unsr16 IS

	signal q3tmp,q4tmp,q7tmp,q8tmp,qbtmp,qctmp : bit;

BEGIN

    U1:unsr4 port map(sl,q4tmp,s0,s1,d3,d2,d1,d0,clk,r,q3tmp,q2,q1, q0);
    U2:unsr4 port map(q3tmp,q8tmp,s0,s1,d7,d6,d5,d4,clk,r,q7tmp,q6,q5,q4tmp);
    U3:unsr4 port map(q7tmp,qctmp,s0,s1,d11,d10,d9,d8,clk,r,qbtmp,q10,q9,q8tmp);
    U4:unsr4 port map(qbtmp,sr,s0,s1,d15,d14,d13,d12,clk,r,q15,q14,q13,qctmp);

    q3  <= q3tmp;
    q4  <= q4tmp;
    q7  <= q7tmp;
    q8  <= q8tmp;
    q11 <= qbtmp;
    q12 <= qctmp;

END archunsr16;
