--
--      Copyright Cypress Semiconductor Corporation, 1995,1996
--	as an unpublished work.
--

--
--	TTL82
--

ENTITY ttl82 IS
	PORT (C0 : IN BIT;
	      A1, A2, B1, B2: IN BIT;
              S1, S2: OUT BIT;
              C2: OUT BIT);
END ttl82;

ARCHITECTURE archttl82 OF ttl82 IS

	signal na1_o, na2_o, na3_o, na4_o:bit;
	signal na5_o, na6_o, na7_o, na8_o:bit;
	signal na9_o, na10_o, na11_o, na12_o:bit;
	signal na13_o, na14_o:bit;
	signal a2i, b2i, fb, fb2: bit;


BEGIN

	a2i <= not a2;
	b2i <= not b2;

        na1_o <= (C0 AND fb) ;
        na2_o <= (a1 AND fb) ;
        na3_o <= (b1 AND fb) ;
	na4_o <= (a1 and b1 and C0) ;
        na5_o <= (a1 AND C0) ;
        na6_o <= (b1 AND C0) ;
        na7_o <= (a1 AND b1) ;

        na8_o <= (fb AND fb2) ;
        na9_o <= (a2i AND fb2) ;
        na10_o <= (b2i AND fb2) ;
	na11_o <= fb and a2i and b2i ;
        na12_o <= (fb AND a2i) ;
        na13_o <= (fb AND b2i) ;
        na14_o <= (a2i AND b2i) ;

	s1 <= na1_o OR na2_o OR na3_o OR na4_o;

	fb <= not (na5_o OR na6_o OR na7_o);

	s2 <= not (na8_o OR na9_o OR na10_o OR na11_o);

	fb2 <= not (na12_o OR na13_o OR na14_o);

	c2 <= fb2;

END archttl82;

--
--	TTL83
--

ENTITY ttl83 IS
	PORT (C0 : IN BIT;
	      A1, A2, A3, A4, B1, B2, B3, B4 : IN BIT;
              S1, S2, S3, S4: OUT BIT;
              C4: OUT BIT);
END ttl83;

ARCHITECTURE archttl83 OF ttl83 IS

COMPONENT fcadd4
PORT (CI : IN BIT;
    A3,A2,A1,A0: IN BIT;
    B3,B2,B1,B0: IN BIT;
    SUM3,SUM2,SUM1,SUM0 : OUT BIT;
    CO: OUT BIT);
END COMPONENT;

BEGIN

	fc:fcadd4 port map(c0,a4,a3,a2,a1,b4,b3,b2,b1,s4,s3,s2,s1,c4);

END archttl83;

--
--	TTL85
--

ENTITY ttl85 IS
        PORT (A0, A1, A2, A3, B0, B1, B2, B3: IN BIT;
 	      albi,aebi,agbi: IN BIT;
 	      alb,aeb,agb: OUT BIT);
END ttl85;


ARCHITECTURE archttl85 OF ttl85 IS
	signal na1_o,na2_o,na3_o,na4_o: bit;
	signal an1_o,an2_o,an3_o,an4_o,an5_o,an6_o,an7_o,an8_o: bit;
	signal no1_o,no2_o,no3_o,no4_o : bit;
BEGIN
        na1_o <= NOT (a0 AND b0) ;
        na2_o <= NOT (a1 AND b1) ;
        na3_o <= NOT (a2 AND b2) ;
        na4_o <= NOT (a3 AND b3) ;

        an1_o <= (na1_o AND a0) ;
        an2_o <= (na1_o AND b0) ;
        an3_o <= (na2_o AND a1) ;
        an4_o <= (na2_o AND b1) ;
        an5_o <= (na3_o AND a2) ;
        an6_o <= (na3_o AND b2) ;
        an7_o <= (na4_o AND a3) ;
        an8_o <= (na4_o AND b3) ;

        no1_o <= NOT (an1_o OR an2_o) ;
        no2_o <= NOT (an3_o OR an4_o) ;
        no3_o <= NOT (an5_o OR an6_o) ;
        no4_o <= NOT (an7_o OR an8_o) ;

	agb <=  not (b3 and na4_o) AND
		not (b2 and na3_o and no4_o) AND
		not (b1 and na2_o and no3_o and no4_o) AND
		not (b0 and na1_o and no2_o and no3_o and no4_o) AND
		not (no1_o and no2_o and no3_o and no4_o and albi) AND
		not (no1_o and no2_o and no3_o and no4_o and aebi);

	aeb <=  no1_o and no2_o and no3_o and no4_o and aebi;

	alb <=  not (a3 and na4_o) AND
		not (a2 and na3_o and no4_o) AND
		not (a1 and na2_o and no3_o and no4_o) AND
		not (a0 and na1_o and no2_o and no3_o and no4_o) AND
		not (no1_o and no2_o and no3_o and no4_o and agbi) AND
		not (no1_o and no2_o and no3_o and no4_o and aebi);

END archttl85;

--
--	TTL148
--

ENTITY ttl148 IS
PORT (EI, I0, I1, I2, I3, I4, I5, I6, I7 : IN BIT;
	A0,A1,A2,GS,EO : OUT BIT);
END ttl148;

ARCHITECTURE archttl148 of ttl148 IS
BEGIN
	EO <= not ((not EI) and I0 and I1 and I2 and I3 and I4 and I5
		and I6 and I7);

	GS <= not(not((not EI) and I0 and I1 and I2 and I3 and I4 and I5
		and I6 and I7) and not EI);

	A0 <= not (
		((not I1) and I2 and I4 and I6 and (not EI)) OR
		((not I3) and I4 and I6 and (not EI)) OR
		((not I5) and I6 and (not EI)) OR
		((not I7) and (not EI))
		);

	A1 <= not (
		((not I2) and I4 and I5 and (not EI)) OR
		((not I3) and I4 and I5 and (not EI)) OR
		((not I6) and (not EI)) OR
		((not I7) and (not EI))
		);

	A2 <= not (
		((not I4) and (not EI)) OR
		((not I5) and (not EI)) OR
		((not I6) and (not EI)) OR
		((not I7) and (not EI))
		);
END archttl148;

ENTITY ttl180 IS
	PORT (A,B,C,D,E,F,G,H, ODDI, EVEI: IN BIT;
	      ODD, EVE: OUT BIT);
END ttl180;

ARCHITECTURE archttl180 of ttl180 is
	signal xum3 : bit;
	signal xum2_1, xum2_2: bit;
BEGIN
	xum2_1 <= (not (A xor B)) xor (not (C xor D));
	xum2_2 <= (not (E xor F)) xor (not (G xor H));
	xum3 <= not (xum2_1 xor xum2_2);

	eve <=  (not (xum3 and oddi))
		and (not (not xum3 and evei));

	odd <=  (not (xum3 and evei))
		and (not (not xum3 and oddi));

END archttl180;

--
--	TTL261
--

ENTITY ttl261 is
	PORT (C : IN BIT;
              B0,B1,B2,B3,B4 : IN BIT;
              M0,M1,M2 : IN BIT;
              Q0,Q1,Q2,Q3,Q4: OUT BIT);
END ttl261;

ARCHITECTURE archttl261 of ttl261 is

	signal c3,c4,c5,c6,c7: bit;
	signal q0_tmp, q1_tmp, q2_tmp, q3_tmp, q4_tmp,
		q5_tmp: bit;

BEGIN

	c3 <= not ( (m0 and m1) or not (m0 or m1) );
	c4 <= ( m0 and m1 );
	c5 <= not ( m1 or m0 );
	c6 <= not ( m2 or not c );
	c7 <=  (( m2 or not c ) and c);

	q0_tmp <=       (b0 and c4 and c6)
		     or (not b0 and c5 and c7)
		     or (b1 and c3 and c6)
		     or (not b1 and c3 and c7)
		     or (q0_tmp and not c);

	q0 <= q0_tmp;

	q1_tmp <=	(b1 and c4 and c6)
		     or (not b1 and c5 and c7)
		     or (b2 and c3 and c6)
		     or (not b2 and c3 and c7)
		     or (q1_tmp and not c);

	q1 <= q1_tmp;
	
	q2_tmp <=	(b2 and c4 and c6)
		     or (not b2 and c5 and c7)
		     or (b3 and c3 and c6)
		     or (not b3 and c3 and c7)
		     or (q2_tmp and not c);

	q2 <= q2_tmp;

	q3_tmp <=	(b3 and c4 and c6)
		    or  (not b3 and c5 and c7)
		    or  (b4 and c3 and c6)
		    or  (not b4 and c3 and c7)
		    or  (q3_tmp and not c);

	q3 <= q3_tmp;

	q4_tmp <=	(b4 and c4 and c6)
		    or	(not b4 and c5 and c7)
		    or  (b4 and c3 and c6)
		    or  (not b4 and c3 and c7)
		    or  (q4_tmp and not c);

	q4 <= not q4_tmp;
		

END archttl261;

--
--	TTL518
--

ENTITY ttl518 IS
PORT (G, Q0, Q1, Q2, Q3, Q4, Q5, Q6, Q7,
	 P0, P1, P2, P3, P4, P5, P6, P7 : IN BIT;
      PQ : OUT BIT);
END ttl518;

ARCHITECTURE archttl518 of ttl518 is
BEGIN
	PQ <= (not G) and
	      (not (Q7 xor P7)) and
	      (not (Q6 xor P6)) and
	      (not (Q5 xor P5)) and
	      (not (Q4 xor P4)) and
	      (not (Q3 xor P3)) and
	      (not (Q2 xor P2)) and
	      (not (Q1 xor P1)) and
	      (not (Q0 xor P0));
END archttl518;

