-- mth34x8.vhd
--
-- Copyright 1993,1994,1995,1996 Cypress Semiconductor as an unpublished work.
--
-- Rev 1.0	09-20-93
-- Rev 1.1	12-06-93
--
-- This is a package that defines math components of length 8.
-- The components are defined and entity architecture pairs
-- follow, suitable for implementation in 34X devices (ie.
-- the architectures render operations in terms of expanders and
-- macrocells). The synthesis_off attribute is used and use of
-- the -xor2 synthesis directive is expected.
--
-- Components:
-- adder	: add_8
-- subtractor	: sub_8
-- grt than(>)	: gt_8
-- less than(<)	: lt_8
-- gt or eq(>=)	: ge_8
-- lt or eq(<=)	: lt_8   
--
--
package math34x8_pkg is

component add_8 port(
  a,b:in bit_vector(7 downto 0);
  s:out bit_vector(7 downto 0)
  );
end component;

 
component sub_8 port(
  a,b:in bit_vector(7 downto 0);
  s:out bit_vector(7 downto 0)
  );
end component;

component gt_8 port(
  a,b:in bit_vector(7 downto 0);
  a_gt_b:out bit
  );
end component;

component lt_8 port(
  a,b:in bit_vector(7 downto 0);
  a_lt_b:out bit
  );
end component;

component ge_8 port(
  a,b:in bit_vector(7 downto 0);
  a_ge_b:out bit
  );
end component;

component le_8 port(
  a,b:in bit_vector(7 downto 0);
  a_le_b:out bit
  );
end component;

end math34x8_pkg;

-- ***************************************************
-- entities and architectures for math components
-- ***************************************************

use work.cypress.all;
entity add_8 is port(
  a,b:in bit_vector(7 downto 0);
  s:out bit_vector(7 downto 0)
  );
end add_8;

--***********************************************
architecture ma34x of add_8 is 
 signal x:bit_vector(1 to 15);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(7 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 14);
 signal car:bit_vector(0 to 1);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 3);

begin
s<=si;

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=not(b(0)) and not(a(0)) and x(1);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(a(0) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=x(2) and x(3);
eq(4)<=b(0) and a(0);
   
-- expanders:
x(2)<=not(not(a(1)) and not(b(1)));
x(3)<=not(a(1) and b(1));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<=(b(0) and a(0)and x(2))
     or (b(1) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and not(b(2)));
x(5)<=not(a(2) and b(2));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(b(0) and a(0) and x(2) and x(4))
     or (b(1) and a(1) and x(4))
     or (b(2) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and not(b(3)));
x(7)<=not(a(3) and b(3));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(b(0) and a(0) and x(2) and x(4) and x(6) and x(7))
     or (b(1) and a(1) and x(4) and x(6) and x(7))
     or (b(2) and a(2) and x(6) and x(7));
em(1)<=a(3) and b(3);

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and not(b(4)));
x(9)<=not(a(4) and b(4));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and b(4))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and not(b(5)));
x(10)<=not(a(5) and b(5));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and b(4) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and b(5));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and not(b(6)));
x(13)<=not(a(6) and b(6));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(b(4) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (b(5) and a(5) and x(12) and x(13));
em(3)<=a(6) and b(6);

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and not(b(7)));
x(15)<=not(a(7) and b(7));

end ma34x;
--***********************************************


--***********************************************
--   sub_n component ****************************
--***********************************************

use work.cypress.all;
entity sub_8 is port(
  a,b:in bit_vector(7 downto 0);
  s:out bit_vector(7 downto 0)
  );
end sub_8;

--***********************************************
architecture ma34x of sub_8 is 
 signal x:bit_vector(1 to 15);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(7 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 14);
 signal car:bit_vector(0 to 1);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 3);

begin
s<=si;

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=not(a(0)) or b(0);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=x(1);
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (x(1) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(x(1) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(x(1) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

end ma34x;
--***********************************************

use work.cypress.all;
entity gt_8 is port(
  a,b:in bit_vector(7 downto 0);
  a_gt_b: out bit
  );
end gt_8;

architecture ma34x of gt_8 is 
 signal x:bit_vector(1 to 15);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(8 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 16);
 signal car:bit_vector(0 to 1);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 3);

begin
a_gt_b<=si(8);

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=b(0) and not(a(0)) and x(1);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=a(0) and not(b(0));
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (not(b(0)) and a(0) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(not(b(0)) and a(0) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(not(b(0)) and a(0) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(not(a(7)) and not(car(1))) or
        (not(a(7)) and b(7)) or
        (b(7) and not(car(1))); -- eqn inverted from lt_
eq(16)<=x(14) and x(15);

end ma34x; -- gt_8
--************************************************

use work.cypress.all;
entity lt_8 is port(
  a,b:in bit_vector(7 downto 0);
  a_lt_b: out bit
  );
end lt_8;

architecture ma34x of lt_8 is 
 signal x:bit_vector(1 to 15);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(8 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 16);
 signal car:bit_vector(0 to 1);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 3);

begin
a_lt_b<=si(8);

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=not(a(0)) or b(0);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=x(1);
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (x(1) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(x(1) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(x(1) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and not(b(7)))
      or (car(1) and x(14));
eq(16)<=x(14) and x(15);

end ma34x; -- lt_8
--************************************************


use work.cypress.all;
entity ge_8 is port(
  a,b:in bit_vector(7 downto 0);
  a_ge_b: out bit
  );
end ge_8;


architecture ma34x of ge_8 is 
 signal x:bit_vector(1 to 15);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(8 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 16);
 signal car:bit_vector(0 to 1);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 3);

begin
a_ge_b<=si(8);

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=not(a(0)) or b(0);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=x(1);
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (x(1) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(x(1) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(x(1) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(not(a(7)) and not(car(1))) or
        (not(a(7)) and b(7)) or
        (b(7) and not(car(1))); -- eqn inverted from lt_
eq(16)<=x(14) and x(15);

end ma34x; -- ge_8
--************************************************

use work.cypress.all;
entity le_8 is port(
  a,b:in bit_vector(7 downto 0);
  a_le_b: out bit
  );
end le_8;


architecture ma34x of le_8 is 
 signal x:bit_vector(1 to 15);
 attribute synthesis_off of x:signal is true;
 signal si:bit_vector(8 downto 0);
 attribute synthesis_off of si:signal is true;
 signal eq:bit_vector(1 to 16);
 signal car:bit_vector(0 to 1);
 attribute synthesis_off of car:signal is true;
 signal em:bit_vector(0 to 3);

begin
a_le_b<=si(8);

-- macrocell:
si(0)<=eq(2) xor eq(1);
eq(1)<=b(0) and not(a(0)) and x(1);
eq(2)<=x(1);
   
-- expander:
x(1)<=not(not(a(0)) and b(0));

-- macrocell:
si(1)<=eq(4) xor eq(3);
eq(3)<=a(0) and not(b(0));
eq(4)<=x(2) and x(3);
   
-- expanders:
x(2)<=not(not(a(1)) and b(1));
x(3)<=not(a(1) and not(b(1)));

-- macrocell:
si(2)<=eq(6) xor eq(5);
eq(5)<= (not(b(0)) and a(0) and x(2))
     or (not(b(1)) and a(1));
eq(6)<=x(4) and x(5);

-- expanders:
x(4)<=not(not(a(2)) and b(2));
x(5)<=not(a(2) and not(b(2)));

-- macrocell:
si(3)<=eq(8) xor eq(7);
eq(7)<=(not(b(0)) and a(0) and x(2) and x(4))
     or (not(b(1)) and a(1) and x(4))
     or (not(b(2)) and a(2));
eq(8)<=x(6) and x(7);

-- expanders:
x(6)<=not(not(a(3)) and b(3));
x(7)<=not(a(3) and not(b(3)));

-- carry macrocell zero:
car(0)<=em(1) xor em(0);
em(0)<=(not(b(0)) and a(0) and x(2) and x(4) and x(6) and x(7))
     or (not(b(1)) and a(1) and x(4) and x(6) and x(7))
     or (not(b(2)) and a(2) and x(6) and x(7));
em(1)<=a(3) and not(b(3));

-- macrocell:
si(4)<=eq(9) xor car(0);
eq(9)<=x(8) and x(9);

-- expanders:
x(8)<=not(not(a(4)) and b(4));
x(9)<=not(a(4) and not(b(4)));

-- macrocell:
si(5)<=eq(11) xor eq(10);
eq(10)<=(a(4) and not(b(4)))
      or (car(0) and x(8));
eq(11)<=x(10) and x(11);

-- expanders:
x(11)<=not(not(a(5)) and b(5));
x(10)<=not(a(5) and not(b(5)));

-- macrocell:
si(6)<=eq(13) xor eq(12);
eq(12)<=(a(4) and not(b(4)) and x(11))
      or (car(0) and x(8) and x(11))
      or (a(5) and not(b(5)));
eq(13)<=x(12) and x(13);

-- expanders:
x(12)<=not(not(a(6)) and b(6));
x(13)<=not(a(6) and not(b(6)));

-- carry macrocell one:
car(1)<=em(3) xor em(2);
em(2)<=(not(b(4)) and a(4) and x(11) and x(12) and x(13))
     or (car(0) and x(8) and x(11) and x(12) and x(13))
     or (not(b(5)) and a(5) and x(12) and x(13));
em(3)<=a(6) and not(b(6));

-- macrocell:
si(7)<=eq(14) xor car(1);
eq(14)<=x(14) and x(15);

-- expanders:
x(14)<=not(not(a(7)) and b(7));
x(15)<=not(a(7) and not(b(7)));

-- macrocell:
si(8)<=eq(16) xor eq(15);
eq(15)<=(a(7) and not(b(7)))
      or (car(1) and x(14));
eq(16)<=x(14) and x(15);

end ma34x; -- le_8
--************************************************

