--
--      Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--        as an unpublished work.
--

library ieee;
use ieee.std_logic_1164.all;

use work.c37xp.all;

ENTITY c37xinp IS
    GENERIC (
	iregister	: ireg_type	:= ireg;
	tCLK	:	time :=  4000 ps;
	tIRD	:	time :=  1000 ps;
	tICOMB	:	time :=  3000 ps;
	tILAT	:	time :=  5500 ps;
	tIRSU	:	time :=  6000 ps;
	tIRH	:	time :=  2000 ps;
	tSOP	:	time :=  1500 ps;
	tORD	:	time :=  1000 ps;
	tOCOMB	:	time :=  3000 ps;
	tOLAT	:	time :=  5500 ps;
	tORSU	:	time :=  4500 ps;
	tORH	:	time :=  0000 ps;
	tOD	:	time :=  1000 ps;
	tXZ	:	time :=  8500 ps;
	tZX	:	time :=  8500 ps;
	tFB	:	time :=  0000 ps;
	tRST	:	time :=  8500 ps;
	tPST	:	time :=  8500 ps;
	tWL	:	time :=  2500 ps;
	tWH	:	time :=  2500 ps;
	tRW	:	time :=  8000 ps;
	tRR	:	time := 10000 ps;
	tPW	:	time :=  8000 ps;
	tPR	:	time := 10000 ps);
    PORT (
	d	: in	std_logic;
	clk	: in	std_logic;
	fb	: out	std_logic);
END c37xinp;

ARCHITECTURE sim OF c37xinp IS
    SIGNAL   	q1		:	std_logic	:= '0';
    SIGNAL	q2		:	std_logic	:= '0';
    SIGNAL	q3		:	std_logic	:= '0';
    SIGNAL	iregq_tis	: 	std_logic	:= '0';
    SIGNAL	iregq_tih	: 	std_logic	:= '0';
    SIGNAL	iregq_twh	: 	std_logic	:= '0';
    SIGNAL	iregq_twl	: 	std_logic	:= '0';
BEGIN
    -- check for clk setup/hold violations
    PROCESS (clk, d)

	VARIABLE dlastev	: TIME := 0 ns;
	VARIABLE clklastev	: TIME := 0 ns;

    BEGIN
	-- only if not combinatorial.
	IF (iregister /= comb) THEN

	    -- setup check
	    IF (clk'event AND (clk = '1')) THEN
		ASSERT ((NOW = 0 ns) OR ((NOW - dlastev) >= Tirsu))
		    REPORT
		    "Setup ERROR ON D: Setting output to unknown" 
		    SEVERITY WARNING;
		IF NOT ((NOW = 0 ns) OR ((NOW - dlastev) >= Tirsu)) THEN
		    iregq_tis <= '1'; 
		    iregq_tis <= transport '0' after 1 ns;
		END IF;
		clklastev := NOW;
	    END IF;

	    -- hold check
	    IF (d'event) then
		ASSERT ((NOW = 0 ns) OR ((NOW - clklastev) >= Tirh))
		    REPORT
		    "Hold ERROR ON D: Setting output to unknown" 
		    SEVERITY WARNING;
		IF NOT ((NOW = 0 ns) OR ((NOW - clklastev) >= Tirh)) THEN
		    iregq_tih <= '1'; 
		    iregq_tih <= transport '0' after 1 ns;
		END IF;
		dlastev := NOW;
	    END IF;
	END IF;
    END PROCESS;

    -- check clk pulse width
    PROCESS (clk)
	VARIABLE clklastevhi,clklastevlo : TIME := 0 ns;
    BEGIN
	-- only if not combinatorial.
	IF (iregister /= comb) THEN

	    IF (clk = '0') THEN
		ASSERT ((NOW = 0 ns) OR ((NOW - clklastevhi) >= Twh))
		    REPORT
		    "Pulse Width (H) ERROR on CLK: Setting output to unknown"
		    SEVERITY WARNING;
		IF NOT ((NOW = 0 ns) OR ((NOW - clklastevhi) >= Twh)) THEN
		    iregq_twh <= '1'; 
		    iregq_twh <= transport '0' after 1 ns;
		END IF;
		clklastevlo := NOW;
	    ELSIF (clk = '1') THEN
	       ASSERT ((NOW = 0 ns) OR ((NOW - clklastevlo) >= Twl))
		    REPORT
		    "Pulse Width (L) ERROR on CLK: Setting output to unknown"
		    SEVERITY WARNING;
		IF NOT ((NOW = 0 ns) OR ((NOW - clklastevlo) >= Twl)) THEN
		    iregq_twl <= '1'; 
		    iregq_twl <= transport '0' after 1 ns;
		END IF;
		clklastevhi := NOW;
	    END IF;
	END IF;
    END PROCESS;

    -- Simulate the Register.
    PROCESS (d, clk, iregq_tis, iregq_tih, iregq_twh, iregq_twl)

    BEGIN
	IF ((iregq_tis = '1') OR (iregq_tih = '1') OR (iregq_twh = '1')
	OR (iregq_twl = '1')) THEN
	    q1 <= 'X' after Tird;
	    q2 <= 'X' after Tird;
	    q3 <= 'X' after Tird;
	ELSIF (clk'event AND (clk = '1')) THEN
	    q1 <= d after Tird;
	    q2 <= q1 after Tird;
	    q3 <= d after Tird;
	ELSIF (clk = '1') THEN
	    q3 <= d after Tilat;
	END IF;
    END PROCESS;

    -- Simulate the Mux.
    PROCESS (q1, q2, q3, d)

    BEGIN
	IF (iregister = comb) THEN
	    fb <= d after Ticomb;
	ELSIF (iregister = ireg) THEN
	    fb <= q1;
	ELSIF (iregister = dblireg) THEN	
	    fb <= q2;
	ELSIF (iregister = iltch) THEN
	    fb <= q3;
	END IF;
    END PROCESS;

END sim;
