--
--	Copyright Cypress Semiconductor Corporation, 1995,1996
--	  as an unpublished work.
--

library ieee;
use ieee.std_logic_1164.all;

PACKAGE mcpartspkg IS

component papbc
    PORT (a			: in    std_logic;
	y			: out   std_logic);
end component;
    
component paincell
    PORT (ip			: in    std_logic;
	ini,iz			: out   std_logic);
end component;
    
component pabicell
    PORT (ie,i1,i2		: in    std_logic;
	iz			: out   std_logic;
	ip			: inout std_logic);
end component;
	
component packcell
    PORT (ip			: in    std_logic;
	ini,iz,ic		: out   std_logic);
end component;

component pafrag_a
    PORT (a1,a2,a3,a4,a5,a6	: in    std_logic;
	az			: out   std_logic);
end component;
    
component pafrag_f
    PORT (f1,f2,f3,f4,f5,f6	: in    std_logic;
	fz			: out   std_logic);
end component;
    
component pafrag_m
    PORT (os,nsi		: in    std_logic;
	b1,b2			: in    std_logic;
	c1,c2			: in    std_logic;
	d1,d2			: in    std_logic;
	e1,e2			: in    std_logic;
	oz,nz			: out   std_logic);
end component;

component pafrag_q
    PORT (qs,qc,qr,qd		: in    std_logic;
	qz			: out   std_logic);
end component;

component palcell
    PORT (qs			: in    std_logic;
	a1,a2,a3,a4,a5,a6	: in    std_logic;
	b1,b2			: in    std_logic;
	c1,c2			: in    std_logic;
	d1,d2			: in    std_logic;
	e1,e2			: in    std_logic;
	f1,f2,f3,f4,f5,f6	: in    std_logic;
	qc,qr			: in    std_logic;
	az,oz,qz,nz,fz		: out   std_logic);
end component;

component ppalcell
    PORT (qs			: in    std_logic;
	a1,a2,a3,a4,a5,a6	: in    std_logic;
	b1,b2			: in    std_logic;
	c1,c2			: in    std_logic;
	d1,d2			: in    std_logic;
	e1,e2			: in    std_logic;
	f1,f2,f3,f4,f5,f6	: in    std_logic;
	qc,qr			: in    std_logic;
	az,oz,qz,nz,fz		: out   std_logic);
end component;

component palogic
    PORT (qs			: in    std_logic;
	a1,a2,a3,a4,a5,a6	: in    std_logic;
	b1,b2			: in    std_logic;
	c1,c2			: in    std_logic;
	d1,d2			: in    std_logic;
	e1,e2			: in    std_logic;
	f1,f2,f3,f4,f5,f6	: in    std_logic;
	qc,qr			: in    std_logic;
	az,oz,qz,nz,fz		: out   std_logic);
end component;

component logico
    PORT (a1,a2,a3,a4,a5,a6	: in    std_logic;
	b1,b2			: in    std_logic;
	c1,c2			: in    std_logic;
	d1,d2			: in    std_logic;
	e1,e2			: in    std_logic;
	f1,f2,f3,f4,f5,f6	: in    std_logic;
	oz			: out   std_logic);
end component;

end mcpartspkg;

---------------------------------------------

--
--	papbc
--

library ieee;
use ieee.std_logic_1164.all;

ENTITY papbc is
    PORT (a			: in    std_logic;
	y			: out   std_logic);
end papbc;

architecture archpapbc of papbc is
begin

    y <= a;

end archpapbc;

--
--	paincell
--

library ieee;
use ieee.std_logic_1164.all;

ENTITY paincell is
    PORT (ip			: in    std_logic;
	ini,iz			: out   std_logic);
end paincell;

architecture archpaincell of paincell is
begin

    ini	<= not(ip);
    iz	<= ip;

end archpaincell;

--
--	pabicell
--
    

library ieee;
use ieee.std_logic_1164.all;

ENTITY pabicell is
    PORT (ie,i1,i2		: in    std_logic;
	iz			: out   std_logic;
	ip			: inout std_logic);
end pabicell;

architecture archpabicell of pabicell is

    signal ory	: std_logic;
component bufoe
    port(
	x	: in std_logic;		-- Logic input to Buffer.
	oe	: in std_logic;		-- Output Enable input.
	y	: inout std_logic;		-- Output pin.
	yfb	: out std_logic		-- Output feed back as std_logic.
    );
end component;
begin

    ory	<= i2 or not(i1);
    u1:bufoe port map ( oe => ie, x => ory, yfb => iz, y => ip);

end archpabicell;

--
--	packcell
--

library ieee;
use ieee.std_logic_1164.all;

ENTITY packcell is
    PORT (ip			: in    std_logic;
	ini,iz,ic		: out   std_logic);
end packcell;

architecture archpackcell of packcell is
begin

    ini	<= not (ip);
    iz	<= ip;
    ic	<= ip;

end archpackcell;

--
--	pafrag_a
--

library ieee;
use ieee.std_logic_1164.all;

ENTITY pafrag_a is
    PORT (a1,a2,a3,a4,a5,a6	: in    std_logic;
	az			: out   std_logic);
end pafrag_a;

architecture archpafrag_a of pafrag_a is
begin

    az	<= a1 and not(a2) and a3 and not(a4) and a5 and not(a6);

end archpafrag_a;

--
--	pafrag_f
--

library ieee;
use ieee.std_logic_1164.all;

ENTITY pafrag_f is
    PORT (f1,f2,f3,f4,f5,f6	: in    std_logic;
	fz			: out   std_logic);
end pafrag_f;

architecture archpafrag_f of pafrag_f is
begin

    fz	<= f1 and not(f2) and f3 and not(f4) and f5 and not(f6);

end archpafrag_f;
    
--
--	pafrag_m
--

library ieee;
use ieee.std_logic_1164.all;

ENTITY pafrag_m is
    PORT (os,nsi		: in    std_logic;
	b1,b2			: in    std_logic;
	c1,c2			: in    std_logic;
	d1,d2			: in    std_logic;
	e1,e2			: in    std_logic;
	oz,nz			: out   std_logic);
end pafrag_m;

architecture archpafrag_m of pafrag_m is

    signal by, cy, dy, ey, mzi, nzi, ozi : std_logic;

begin

    by	<= b1 and not(b2);
    cy	<= c1 and not(c2);
    dy	<= d1 and not(d2);
    ey	<= e1 and not(e2);

    mzi	<= (cy and nsi) or (by and not(nsi));
    nzi	<= (ey and nsi) or (dy and not(nsi));
    nz	<= nzi;

    ozi	<= (nzi and os) or (mzi and not(os));
    oz	<= ozi;

end archpafrag_m;

--
--	pafrag_q
--

library ieee;
use ieee.std_logic_1164.all;

ENTITY pafrag_q is
    PORT (qs,qc,qr,qd		: in    std_logic;
	qz			: out   std_logic);
end pafrag_q;

architecture archpafrag_q of pafrag_q is
component dsrff
    port(
	d	: in std_logic;		-- D input to register.
	s	: in std_logic;		-- Set input Async.
	r	: in std_logic;		-- Reset input Async.
	clk	: in std_logic;		-- Positive edge Clock.
	q	: out std_logic		-- Output from register.
    );
end component;
begin

    u1 : dsrff port map ( s => qs, clk => qc, r => qr, d => qd, q => qz);

end archpafrag_q;

--
--	palcell
--

library ieee;
use ieee.std_logic_1164.all;

use work.mcpartspkg.all;
ENTITY palcell is
    PORT (qs			: in    std_logic;
	a1,a2,a3,a4,a5,a6	: in    std_logic;
	b1,b2			: in    std_logic;
	c1,c2			: in    std_logic;
	d1,d2			: in    std_logic;
	e1,e2			: in    std_logic;
	f1,f2,f3,f4,f5,f6	: in    std_logic;
	qc,qr			: in    std_logic;
	az,oz,qz,nz,fz		: out   std_logic);
END palcell;

architecture archpalcell of palcell is
    signal aztmp, fztmp, oztmp : std_logic;
begin

    u1: pafrag_a port map (a1,a2,a3,a4,a5,a6,aztmp);
    u2: pafrag_m port map (aztmp,fztmp,b1,b2,c1,c2,d1,d2,e1,e2,oztmp,nz);
    u3: pafrag_f port map (f1,f2,f3,f4,f5,f6,fztmp);
    u4: pafrag_q port map (qs,qc,qr,oztmp,qz);

    az <= aztmp;
    fz <= fztmp;
    oz <= oztmp;

end archpalcell;

--
--	ppalcell
--

library ieee;
use ieee.std_logic_1164.all;

use work.mcpartspkg.all;
ENTITY ppalcell is
    PORT (qs			: in    std_logic;
	a1,a2,a3,a4,a5,a6	: in    std_logic;
	b1,b2			: in    std_logic;
	c1,c2			: in    std_logic;
	d1,d2			: in    std_logic;
	e1,e2			: in    std_logic;
	f1,f2,f3,f4,f5,f6	: in    std_logic;
	qc,qr			: in    std_logic;
	az,oz,qz,nz,fz		: out   std_logic);
END ppalcell;

architecture archppalcell of ppalcell is
begin

    u1: palcell port map (qs,a1,a2,a3,a4,a5,a6,b1,b2,c1,c2,d1,d2,e1,e2,
				f1,f2,f3,f4,f5,f6,qc,qr,az,oz,qz,nz,fz);

end archppalcell;

--
--	palogic
--

library ieee;
use ieee.std_logic_1164.all;

use work.mcpartspkg.all;
ENTITY palogic is
    PORT (qs			: in    std_logic;
	a1,a2,a3,a4,a5,a6	: in    std_logic;
	b1,b2			: in    std_logic;
	c1,c2			: in    std_logic;
	d1,d2			: in    std_logic;
	e1,e2			: in    std_logic;
	f1,f2,f3,f4,f5,f6	: in    std_logic;
	qc,qr			: in    std_logic;
	az,oz,qz,nz,fz		: out   std_logic);
END palogic;

architecture archpalogic of palogic is
begin

    u1: palcell port map (qs,a1,a2,a3,a4,a5,a6,b1,b2,c1,c2,d1,d2,e1,e2,
				f1,f2,f3,f4,f5,f6,qc,qr,az,oz,qz,nz,fz);

end archpalogic;

--
--	logico
--

library ieee;
use ieee.std_logic_1164.all;

use work.mcpartspkg.all;
ENTITY logico is
    PORT (a1,a2,a3,a4,a5,a6	: in    std_logic;
	b1,b2			: in    std_logic;
	c1,c2			: in    std_logic;
	d1,d2			: in    std_logic;
	e1,e2			: in    std_logic;
	f1,f2,f3,f4,f5,f6	: in    std_logic;
	oz			: out   std_logic);
END logico;

architecture archlogico of logico is

    signal az,fz : std_logic;

begin

    u1: pafrag_a port map (a1,a2,a3,a4,a5,a6,az);
    u2: pafrag_m port map (az,fz,b1,b2,c1,c2,d1,d2,e1,e2,oz,open);
    u3: pafrag_f port map (f1,f2,f3,f4,f5,f6,fz);

end archlogico;
