--      lpmpkg.vhp							      --
--									      --
--      Copyright Cypress Semiconductor Corporation, 1995,1996		      --
--        as an unpublished work.					      --
--									      --
--------------------------------------------------------------------------------
--			LPM Component Declarations			      --
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

package lpmpkg is

    type bvm		is array (integer range <>,integer range <>) of std_logic;
    type Shift_Type	is (LPM_LOGICAL, LPM_ROTATE, LPM_ARITHMETIC);
    type Repre_Type	is (LPM_SIGNED, LPM_UNSIGNED, LPM_GRAY);
    type Truth_Type	is (LPM_F, LPM_FD, LPM_FR, LPM_FDR);
    type CtDir_Type	is (LPM_NO_DIR, LPM_UP, LPM_DOWN);
    type Arith_Type	is (LPM_NO_TYP, LPM_ADD, LPM_SUB);
    type Fflop_Type	is (LPM_DFF,LPM_TFF);
    type ShDir_Type	is (LPM_LEFT,LPM_RIGHT);
    type goal_type	is (speed,area,combinatorial);
    type port_state_type is (ps_tied_low,ps_tied_high,ps_open,ps_connected);

    signal zero : std_logic := '0' ;
    signal one : std_logic := '1' ;

    component Mcnstnt
	generic(lpm_width	: positive;
		lpm_cvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(	result		: out	std_logic_vector(lpm_width - 1 downto 0));
    end component;
    component Minv
	generic(lpm_width	: positive;
		lpm_hint	: goal_type := speed);
	port(	data		: in	std_logic_vector(lpm_width - 1 downto 0);
		result		: out	std_logic_vector(lpm_width - 1 downto 0));
    end component;
    component Mand
	generic(lpm_width	: positive;
		lpm_size	: positive;
		lpm_hint	: goal_type := speed;
		lpm_data_pol    : string := "";
		lpm_result_pol  : string := "");
	port(	data		: in	std_logic_vector((lpm_width*lpm_size)-1
								    downto 0);
		result		: out	std_logic_vector(lpm_width - 1 downto 0));
    end component;
    component Mor
	generic(lpm_width	: positive;
		lpm_size	: positive;
		lpm_hint	: goal_type := speed;
		lpm_data_pol    : string := "";
		lpm_result_pol  : string := "");
	port(	data		: in	std_logic_vector((lpm_width*lpm_size)-1
								    downto 0);
		result		: out	std_logic_vector(lpm_width - 1 downto 0));
    end component;
    component Mxor
	generic(lpm_width	: positive;
		lpm_size	: positive;
		lpm_hint	: goal_type := speed;
		lpm_data_pol	: string := "";
		lpm_result_pol	: string := "");
	port(	data		: in	std_logic_vector((lpm_width*lpm_size)-1
								    downto 0);
		result		: out	std_logic_vector(lpm_width - 1 downto 0));
    end component;
    component Mbustri
	generic(lpm_width	: positive;
		lpm_hint	: goal_type := speed);
	port(	tridata		: inout	std_logic_vector(lpm_width -1 downto 0);
		data		: in	std_logic_vector(lpm_width - 1 downto 0);
		enabletr	: in	std_logic;
		enabledt	: in	std_logic;
		result		: out	std_logic_vector(lpm_width - 1 downto 0));
    end component;
    component Mmux
	generic(lpm_width	: positive;
		lpm_size	: positive;
		lpm_widths	: positive;
		lpm_hint	: goal_type := speed);
	port(	data		: in	std_logic_vector((lpm_width*lpm_size)-1
								 downto 0);
		sel		: in	std_logic_vector(lpm_widths -1 downto 0);
		result		: out	std_logic_vector(lpm_width - 1 downto 0));
    end component;
    component Mdecode
	generic(lpm_width	: positive;
		lpm_decodes	: positive;
		lpm_hint	: goal_type := speed);
	port(	data		: in	std_logic_vector(lpm_width - 1 downto 0);
		enable		: in	std_logic;
		eq		: out	std_logic_vector(lpm_decodes-1 downto 0));
    end component;
    component Mclshift
	generic(lpm_width	: positive;
		lpm_widthdist	: natural := 0;
		lpm_shifttype	: shift_type := lpm_logical;
		lpm_hint	: goal_type := speed);
	port(	data		: in	std_logic_vector(lpm_width - 1 downto 0);
		distance	: in	std_logic_vector(lpm_widthdist - 1 downto 0);
		direction 	: in	std_logic;
		result		: out	std_logic_vector(lpm_width - 1 downto 0);
		overflow	: out	std_logic;
		underflow	: out	std_logic);
    end component;
    component Madd_sub
	generic(lpm_width	: positive;
		lpm_representation : repre_type := lpm_unsigned;
		lpm_direction	: arith_type := lpm_no_typ;
		lpm_hint	: goal_type := speed);
	port(	dataA		: in	std_logic_vector(lpm_width - 1 downto 0);
		dataB		: in	std_logic_vector(lpm_width - 1 downto 0);
		cin		: in	std_logic;
		add_sub		: in	std_logic;
		result		: out	std_logic_vector(lpm_width - 1 downto 0);
		cout		: out	std_logic;
		overflow	: out   std_logic);
    end component;
    component Mcompare
	generic(lpm_width	: positive;
		lpm_representation : repre_type := lpm_unsigned;
		lpm_hint	: goal_type := speed);
	port(	dataA		: in	std_logic_vector(lpm_width - 1 downto 0);
		dataB		: in	std_logic_vector(lpm_width - 1 downto 0);
		alb		: out	std_logic;
		aeb		: out	std_logic;
		agb		: out	std_logic;
		ageb		: out   std_logic;
		aleb		: out   std_logic;
		aneb		: out   std_logic);
    end component;
    component Mmult
	generic(lpm_widthA	: positive;
		lpm_widthB	: positive;
		lpm_widthS	: natural := 0;
		lpm_widthP	: positive;
		lpm_representation : repre_type := lpm_unsigned;
		lpm_hint	: goal_type := speed);
	port(	dataA		: in	std_logic_vector(lpm_widthA - 1 downto 0);
		dataB		: in	std_logic_vector(lpm_widthB - 1 downto 0);
		sum		: in	std_logic_vector(lpm_widthS - 1 downto 0);
		result		: out	std_logic_vector(lpm_widthP - 1 downto 0));
    end component;
    component Mcounter
	generic(lpm_width	: positive;
		lpm_direction	: ctdir_type := lpm_no_dir;
		lpm_avalue	: string := "";
		lpm_svalue	: string := "";
		lpm_pvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(	data		: in	std_logic_vector(lpm_width - 1 downto 0);
		clock		: in	std_logic;
		clk_en		: in	std_logic;
		cnt_en		: in	std_logic;
		updown		: in	std_logic;
		q		: out	std_logic_vector(lpm_width - 1 downto 0);
		aset		: in	std_logic;
		aclr		: in	std_logic;
		aload		: in	std_logic;
		sset		: in	std_logic;
		sclr		: in	std_logic;
		sload		: in	std_logic;
		testenab	: in	std_logic;
		testin		: in	std_logic;
		testout		: out	std_logic);
    end component;
    component Mlatch
	generic(lpm_width	: positive;
		lpm_avalue	: string := "";
		lpm_pvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(	data		: in	std_logic_vector(lpm_width - 1 downto 0);
		gate		: in	std_logic;
		q		: out	std_logic_vector(lpm_width - 1 downto 0);
		aset		: in	std_logic;
		aclr		: in	std_logic);
    end component;
    component Mdff
	generic(lpm_width	: positive;
		lpm_avalue	: string := "";
		lpm_svalue	: string := "";
		lpm_pvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(	data		: in	std_logic_vector(lpm_width - 1 downto 0);
		clock		: in	std_logic;
		enable		: in	std_logic;
		shiftin		: in	std_logic;
		shiften		: in	std_logic;
		q		: out	std_logic_vector(lpm_width - 1 downto 0);
		shiftout	: out	std_logic;
		aset		: in	std_logic;
		aclr		: in	std_logic;
		sset		: in	std_logic;
		sclr		: in	std_logic;
		testenab	: in	std_logic;
		testin		: in	std_logic;
		testout		: out	std_logic);
    end component;
    component Mshiftreg
	generic(lpm_width	: positive;
		lpm_direction	: shdir_type:= lpm_left;
		lpm_avalue	: string := "";
		lpm_svalue	: string := "";
		lpm_pvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(   data		: in	std_logic_vector((lpm_width-1) downto 0);
		clock		: in	std_logic;
		enable		: in	std_logic;
		shiftin		: in	std_logic;
		load		: in	std_logic;
		q		: out	std_logic_vector((lpm_width-1) downto 0);
		shiftout	: out	std_logic;
		aset		: in	std_logic;
		aclr		: in	std_logic;
		sset		: in	std_logic;
		sclr		: in	std_logic;
		testenab	: in	std_logic;
		testin		: in	std_logic;
		testout		: out	std_logic);
    end component;
    component Mtff
	generic(lpm_width	: positive;
		lpm_avalue	: string := "";
		lpm_svalue	: string := "";
		lpm_pvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(	data		: in	std_logic_vector(lpm_width - 1 downto 0);
		clock		: in	std_logic;
		enable		: in	std_logic;
		q		: out	std_logic_vector(lpm_width - 1 downto 0);
		aset		: in	std_logic;
		aclr		: in	std_logic;
		aload		: in	std_logic;
		sset		: in	std_logic;
		sclr		: in	std_logic;
		sload		: in	std_logic;
		testenab	: in	std_logic;
		testin		: in	std_logic;
		testout		: out	std_logic);
    end component;
    component Mff
	generic(lpm_width	: positive;
		lpm_fftype	: fflop_type := lpm_dff;
		lpm_avalue	: string := "";
		lpm_svalue	: string := "";
		lpm_pvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(   data		: in	std_logic_vector((lpm_width-1) downto 0);
		clock		: in	std_logic;
		enable		: in	std_logic;
		q		: out	std_logic_vector((lpm_width-1) downto 0);
		aset		: in	std_logic;
		aclr		: in	std_logic;
		aload		: in	std_logic;
		sset		: in	std_logic;
		sclr		: in	std_logic;
		sload		: in	std_logic;
		testenab	: in	std_logic;
		testin		: in	std_logic;
		testout		: out	std_logic);
    end component;
    component MGND
	generic(lpm_width	: positive);
	port(	X		: out	std_logic_vector(lpm_width - 1 downto 0));
    end component;
    component MVCC
	generic(lpm_width	: positive);
	port(	X		: out	std_logic_vector(lpm_width - 1 downto 0));
    end component;
    component Mbuf
	generic(lpm_width	: positive;
		lpm_hint	: goal_type := speed);
	port(	data		: in	std_logic_vector(lpm_width - 1 downto 0);
		result		: out	std_logic_vector(lpm_width - 1 downto 0));
    end component;
end lpmpkg;

