--
--      Copyright Cypress Semiconductor Corporation, 1995,1996
--	as an unpublished work.
--

--
--	MCOMPARE
--

library ieee;
use ieee.std_logic_1164.all;

package Mcompare_local is
    type bvm		is array (integer range <>,integer range <>) of std_logic;
    type Shift_Type	is (LPM_LOGICAL, LPM_ROTATE, LPM_ARITHMETIC);
    type Repre_Type	is (LPM_SIGNED, LPM_UNSIGNED, LPM_GRAY);
    type Truth_Type	is (LPM_F, LPM_FD, LPM_FR, LPM_FDR);
    type CtDir_Type	is (LPM_NO_DIR, LPM_UP, LPM_DOWN);
    type Arith_Type	is (LPM_NO_TYP, LPM_ADD, LPM_SUB);
    type Fflop_Type	is (LPM_DFF,LPM_TFF);
    type ShDir_Type	is (LPM_LEFT,LPM_RIGHT);
    type goal_type	is (speed,area,combinatorial);
    type port_state_type is (ps_tied_low,ps_tied_high,ps_open,ps_connected);

    signal zero : std_logic := '0' ;
    signal one : std_logic := '1' ;

component buf
    port(
	x	: in std_logic;		-- Input to Buffer.
	y	: out std_logic		-- Output from Buffer.
    );
end component;
end Mcompare_local ;

library ieee;
use ieee.std_logic_1164.all;

use work.Mcompare_local.all;

entity Mcompare is
    generic(lpm_width	: positive;
	    lpm_representation	: repre_type;
	    lpm_hint	: goal_type);
    port(   dataA	: in	std_logic_vector((lpm_width-1) downto 0);
	    dataB	: in	std_logic_vector((lpm_width-1) downto 0);
	    alb		: out	std_logic;
	    aeb		: out	std_logic;
	    agb		: out	std_logic;
	    ageb	: out   std_logic;
	    aleb	: out   std_logic;
	    aneb	: out   std_logic);
end Mcompare;

architecture archlpmstd of Mcompare is

    constant C : integer := 8;
    constant K : integer := 3;
    signal xnor_array,lt,eq,gt : std_logic_vector(dataA'high downto 0);
    signal eqi  : std_logic_vector((dataA'high/C) downto 0);
    signal aeqb : std_logic_vector(((dataA'high/C) + 1) downto 0);
    signal lti, gti  : std_logic_vector((dataA'high/K) downto 0);
    signal albi,agbi : std_logic_vector(((dataA'high/K) + 1) downto 0);

begin

    xnor_array <= NOT(dataA XOR dataB);

    aeqb(0) <= '1';
    GEQ: for j in 0 to (xnor_array'high/C) generate
	eq(j*C) <= xnor_array(j*C);
	C1: for i in ((j*C)+1) to ((j*C)+(C-1)) generate
	    C2: if (i < xnor_array'length) generate
		eq(i) <= eq(i-1) AND xnor_array(i);
	    end generate;
	end generate;
	C3: if (j /= (xnor_array'high/C)) generate
	    U1: buf port map (eq((j*C)+(C-1)),eqi(j));
	end generate;
	C4: if (j  = (xnor_array'high/C)) generate
	    C5: if ((xnor_array'high rem C) <  (C-j)) generate
		eqi(j) <= eq(xnor_array'high);
	    end generate;
	    C6: if ((xnor_array'high rem C) >= (C-j)) generate
		U1: buf port map (eq(lpm_width-1),eqi(j));
	    end generate;
	end generate;
	aeqb(j+1) <= aeqb(j) AND eqi(j);
    end generate;
    aeb <= aeqb(aeqb'high);
    aneb <= NOT aeqb(aeqb'high);

    albi(albi'high) <= '0';
    agbi(agbi'high) <= '0';
    GNE: for j in (xnor_array'high/K) downto 0 generate
	lt(j*K) <= dataB(j*K) AND NOT (dataA(j*K) AND dataB(j*K));
	gt(j*K) <= dataA(j*K) AND NOT (dataA(j*K) AND dataB(j*K));
	C1: for i in ((j*K)+1) to ((j*K)+(K-1)) generate
	    C2: if (i < xnor_array'length) generate
		lt(i) <= (dataB(i) AND NOT (dataA(i) AND dataB(i))) OR
			    (lt(i-1) AND xnor_array(i));
		gt(i) <= (dataA(i) AND NOT (dataA(i) AND dataB(i))) OR
			    (gt(i-1) AND xnor_array(i));
	    end generate;
	end generate;
	C3: if (j /= (xnor_array'high/K)) generate
	    U1: buf port map (lt((j*K)+(K-1)),lti(j));
	    U2: buf port map (gt((j*K)+(K-1)),gti(j));
	end generate;
	C4: if (j  = (xnor_array'high/K)) generate
	    C5: if ((xnor_array'high rem K) <  (K-j)) generate
		lti(j) <= lt(xnor_array'high);
		gti(j) <= gt(xnor_array'high);
	    end generate;
	    C6: if ((xnor_array'high rem K) >= (K-j)) generate
		U1: buf port map (lt(lpm_width-1),lti(j));
		U2: buf port map (gt(lpm_width-1),gti(j));
	    end generate;
	end generate;
	albi(j) <= albi(j+1) OR (lti(j) AND NOT agbi(j+1));
	agbi(j) <= agbi(j+1) OR (gti(j) AND NOT albi(j+1));
    end generate;
    alb  <= albi(0);
    aleb <= NOT agbi(0);
    agb  <= agbi(0);
    ageb <= NOT albi(0);

end archlpmstd;
