--
--      Copyright Cypress Semiconductor Corporation, 1995,1996
--	as an unpublished work.
--

library ieee;
use ieee.std_logic_1164.all;

package Mgates_local is
    type bvm		is array (integer range <>,integer range <>) of std_logic;
    type Shift_Type	is (LPM_LOGICAL, LPM_ROTATE, LPM_ARITHMETIC);
    type Repre_Type	is (LPM_SIGNED, LPM_UNSIGNED, LPM_GRAY);
    type Truth_Type	is (LPM_F, LPM_FD, LPM_FR, LPM_FDR);
    type CtDir_Type	is (LPM_NO_DIR, LPM_UP, LPM_DOWN);
    type Arith_Type	is (LPM_NO_TYP, LPM_ADD, LPM_SUB);
    type Fflop_Type	is (LPM_DFF,LPM_TFF);
    type ShDir_Type	is (LPM_LEFT,LPM_RIGHT);
    type goal_type	is (speed,area,combinatorial);
    type port_state_type is (ps_tied_low,ps_tied_high,ps_open,ps_connected);

    signal zero : std_logic := '0' ;
    signal one : std_logic := '1' ;

    function port_state (arg : std_logic) return port_state_type;
    function connects (x : std_logic; i : integer) return integer;
    function max (a,b : natural) return natural;
    function vectorize (a, size : natural) RETURN std_logic_vector;
    function restring (a : string; w : positive) return std_logic_vector;
    component Mand
	generic(lpm_width	: positive;
		lpm_size	: positive;
		lpm_hint	: goal_type := speed;
		lpm_data_pol    : string := "";
		lpm_result_pol  : string := "");
	port(	data		: in	std_logic_vector((lpm_width*lpm_size)-1
								    downto 0);
		result		: out	std_logic_vector(lpm_width - 1 downto 0));
    end component;
    component Mor
	generic(lpm_width	: positive;
		lpm_size	: positive;
		lpm_hint	: goal_type := speed;
		lpm_data_pol    : string := "";
		lpm_result_pol  : string := "");
	port(	data		: in	std_logic_vector((lpm_width*lpm_size)-1
								    downto 0);
		result		: out	std_logic_vector(lpm_width - 1 downto 0));
    end component;
component buf
    port(
	x	: in std_logic;		-- Input to Buffer.
	y	: out std_logic		-- Output from Buffer.
    );
end component;
component bufoe
    port(
	x	: in std_logic;		-- Logic input to Buffer.
	oe	: in std_logic;		-- Output Enable input.
	y	: inout std_logic;		-- Output pin.
	yfb	: out std_logic		-- Output feed back as std_logic.
    );
end component;
component triout
    port(
	x	: in std_logic;		-- Logic input to Buffer.
	oe	: in std_logic;		-- Output Enable input.
	y	: out std_logic		-- Output pin.
    );
end component;
component dsrff
    port(
	d	: in std_logic;		-- D input to register.
	s	: in std_logic;		-- Set input Async.
	r	: in std_logic;		-- Reset input Async.
	clk	: in std_logic;		-- Positive edge Clock.
	q	: out std_logic		-- Output from register.
    );
end component;
component tsrff
    port(
	t	: in std_logic;		-- T input to register.
	s	: in std_logic;		-- Set input Async.
	r	: in std_logic;		-- Reset input Async.
	clk	: in std_logic;		-- Positive edge Clock.
	q	: out std_logic		-- Output from register.
    );
end component;
end Mgates_local;

package body Mgates_local is
    function port_state (arg : std_logic) return port_state_type is
	variable result : port_state_type := ps_connected;
    begin

	result := arg'port_state_att;

	return result;
    end port_state;

    function connects (x : std_logic; i : integer) return integer is
	variable result : integer := i;
    begin
	if (port_state(x) = ps_connected) then
	    result := result - 1;
	end if;
	return (result);
    end connects;

    function max (a,b : natural) return natural is
    begin
	if (a < b) then return b; else return a; end if;
    end max;

    function restring (a : string; w : positive) return std_logic_vector is
	variable result : std_logic_vector (w-1 downto 0) := (others => '1');
	variable indx : natural;
    begin

	if (a'length = 0) then
	    return result;
	elsif (a'length /= w) then
	    Assert (false)
	    Report "RESTRING: Source string length /= target vector width."
	    Severity Failure;
	    return result;
	else
	    indx := 0;
	    for i in a'reverse_range loop
		case a(i) is
		when '1' => result(indx) := '1';
		when '0' => result(indx) := '0';

		when '-' => result(indx) := '-';

		when others => Assert (false)

		    Report "Invalid char. literal (must be '1', '0', or '-').  Defauting to  '0'."

		    Severity Warning;
		    result(indx) := '0';
		end case;
		indx := indx + 1;
	    end loop;
	    return result;
	end if;
    end restring;
    function vectorize (a, size : natural) return std_logic_vector is
	variable result : std_logic_vector(size-1 downto 0) := (others => '0');
	variable b : natural;
	variable newsize : integer := size;
    begin
	if (newsize > 32) then 
	    newsize := 32;
	    assert (false)
	    report "VECTORIZE: Widths > 32 not allowed, vector truncated"
	    severity warning;
	end if;
	if (newsize <= 31) then
	    for i in result'reverse_range loop
		if ((a/(2**i)) mod 2) = 1 then
		    result(i) := '1';
		end if;
	    end loop;
	end if;
	if (newsize = 32) then
	    b := a/2;
	    if ((a mod 2) = 1) then
		result(result'low) := '1';
	    end if;
	    for i in result'low to (result'high - 1) loop
		if ((b/(2**i)) mod 2) = 1 then
		    result(i+1) := '1';
		end if;
	    end loop;
	end if;
	if newsize >= 31 then
	    if ((2**(newsize-2)) < (a/4)) then
		assert (false)
		report
		"VECTORIZE: Width too small to hold integer, vector truncated."
		severity warning;
	    end if;
	else
	    if ((2**newsize) < a) then
		assert (false)
		report
		"VECTORIZE: Width too small to hold integer, vector truncated."
		severity warning;
	    end if;
	end if;
	return result;
    end vectorize;
end Mgates_local;

--
--	MXOR
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mgates_local.all;
entity Mxor is
    generic(lpm_width		: positive;
	    lpm_size		: positive;
	    lpm_hint		: goal_type;
	    lpm_data_pol	: string;
	    lpm_result_pol	: string);
    port(   data	: in	std_logic_vector(((lpm_width*lpm_size)-1) downto 0);
	    result	: out	std_logic_vector((lpm_width-1) downto 0));
end Mxor;

architecture archlpmstd of Mxor is

    signal d_pol : std_logic_vector(((lpm_width*lpm_size)-1) downto 0);
    signal r_pol : std_logic_vector(result'range);

begin

    d_pol <= restring(lpm_data_pol,lpm_width*lpm_size);
    r_pol <= restring(lpm_result_pol,lpm_width);

    process (data)
	variable tmp : std_logic_vector(((lpm_width*lpm_size)-1) downto 0);
    begin
	for n in 0 to lpm_width-1 loop
	    tmp(n*lpm_size) := data(n) XOR NOT d_pol(n);
	    for m in 1 to lpm_size-1 loop
	        tmp((n*lpm_size)+m) := tmp((n*lpm_size)+m-1) XOR 
		    (data((m*lpm_width)+n) XOR NOT d_pol((m*lpm_width)+n));
	    end loop;
	    result(n) <= tmp(((n+1)*lpm_size)-1) XOR NOT r_pol(n);
	end loop;
    end process;

end archlpmstd;

--
--	MOR
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mgates_local.all;

entity Mor is
    generic(lpm_width		: positive;
	    lpm_size		: positive;
	    lpm_hint		: goal_type;
	    lpm_data_pol	: string;
	    lpm_result_pol	: string);
    port(   data	: in	std_logic_vector(((lpm_width*lpm_size)-1) downto 0);
	    result	: out	std_logic_vector((lpm_width-1) downto 0));
end Mor;

architecture archlpmstd of Mor is

    signal d_pol : std_logic_vector(((lpm_width*lpm_size)-1) downto 0);
    signal r_pol : std_logic_vector(result'range);

begin

    d_pol <= restring(lpm_data_pol,lpm_width*lpm_size);
    r_pol <= restring(lpm_result_pol,lpm_width);

    process (data)
	variable tmp : std_logic_vector(((lpm_width*lpm_size)-1) downto 0);
    begin
	for n in 0 to lpm_width-1 loop
	    tmp(n*lpm_size) := data(n) XOR NOT d_pol(n);
	    for m in 1 to lpm_size-1 loop
	        tmp((n*lpm_size)+m) := tmp((n*lpm_size)+m-1) OR 
		    (data((m*lpm_width)+n) XOR NOT d_pol((m*lpm_width)+n));
	    end loop;
	    result(n) <= tmp(((n+1)*lpm_size)-1) XOR NOT r_pol(n);
	end loop;
    end process;

end archlpmstd;

--
--	Expandable VCC
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mgates_local.all;
entity MVCC is
    generic(lpm_width		: positive);
    port(	X		: out	std_logic_vector((lpm_width - 1) downto 0));
end MVCC;

architecture archbit of mvcc is
begin

    X <= (OTHERS => '1');

end archbit;

--
--	MINV
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mgates_local.all;

entity Minv is
    generic(lpm_width	: positive;
	    lpm_hint	: goal_type);
    port(   data	: in	std_logic_vector((lpm_width-1) downto 0);
	    result	: out	std_logic_vector((lpm_width-1) downto 0));
end Minv;

architecture archlpmstd of Minv is
begin

    result <= NOT data;

end archlpmstd;

--
--	Expandable GND
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mgates_local.all;
entity MGND is
    generic(lpm_width		: positive);
    port(	X		: out	std_logic_vector ((lpm_width - 1) downto 0));
end MGND;

architecture archbit of mgnd is
begin

    X <= (OTHERS => '0');

end archbit;

--
--	Mcnstnt
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mgates_local.all;

entity Mcnstnt is
    generic(lpm_width	: positive;
	    lpm_cvalue	: string;
	    lpm_hint	: goal_type);
    port(   result	: out	std_logic_vector((lpm_width-1) downto 0));
end Mcnstnt;

architecture archlpmstd of Mcnstnt is
begin

    result <= restring(lpm_cvalue,lpm_width);

end archlpmstd;

--
--	MAND
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mgates_local.all;

entity Mand is
    generic(lpm_width		: positive;
	    lpm_size		: positive;
	    lpm_hint		: goal_type;
	    lpm_data_pol	: string;
	    lpm_result_pol	: string);
    port(   data	: in	std_logic_vector(((lpm_width*lpm_size)-1) downto 0);
	    result	: out	std_logic_vector((lpm_width-1) downto 0));
end Mand;

architecture archlpmstd of Mand is

    signal d_pol : std_logic_vector(((lpm_width*lpm_size)-1) downto 0);
    signal r_pol : std_logic_vector(result'range);

begin

    d_pol <= restring(lpm_data_pol,lpm_width*lpm_size);
    r_pol <= restring(lpm_result_pol,lpm_width);

    process (data)
	variable tmp : std_logic_vector(((lpm_width*lpm_size)-1) downto 0);
    begin
	for n in 0 to lpm_width-1 loop
	    tmp(n*lpm_size) := data(n) XOR NOT d_pol(n);
	    for m in 1 to lpm_size-1 loop
	        tmp((n*lpm_size)+m) := tmp((n*lpm_size)+m-1) AND 
		    (data((m*lpm_width)+n) XOR NOT d_pol((m*lpm_width)+n));
	    end loop;
	    result(n) <= tmp(((n+1)*lpm_size)-1) XOR NOT r_pol(n);
	end loop;
    end process;

end archlpmstd;

--
--	MDECODE
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mgates_local.all;

entity Mdecode is
    generic(lpm_width	: positive;
	    lpm_decodes	: positive;
	    lpm_hint	: goal_type);
    port(   data	: in  std_logic_vector((lpm_width-1) downto 0);
	    enable	: in  std_logic;
	    eq		: out std_logic_vector((lpm_decodes-1) downto 0));
end Mdecode;

architecture archlpmstd of Mdecode is
begin

    G0: for i in eq'low to eq'high generate
	eq(i) <= '1' when
	    ((enable = '1') AND (vectorize(i,data'length) = data))
	else '0';
    end generate;

end archlpmstd;

--
--	MMUX
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mgates_local.all;

entity Mmux is
    generic(lpm_width	: positive;
	    lpm_size	: positive;
	    lpm_widths	: positive;
	    lpm_hint	: goal_type);
    port(   data	: in	std_logic_vector(((lpm_width*lpm_size)-1) downto 0);
	    sel		: in	std_logic_vector((lpm_widths-1) downto 0);
	    result	: out	std_logic_vector((lpm_width-1) downto 0));
end Mmux;

architecture archlpmstd of Mmux is
begin

    process (data,sel)
	variable tmp : std_logic_vector((lpm_width-1) downto 0);
    begin
	tmp := (others => '0');
	for selval in 0 to ((2 ** lpm_widths)-1) loop
	    if (((lpm_width*(selval+1))-1) <= data'high) AND
	       (vectorize(selval, lpm_widths) = sel) then
		tmp:=data(((lpm_width*(selval+1))-1) downto (lpm_width*selval));
	    end if;
	end loop;
	result <= tmp;
    end process;

end archlpmstd;

--
--	MBUF
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mgates_local.all;

entity Mbuf is
    generic(	lpm_width	: positive;
		lpm_hint	: goal_type);
    port(	data		: in	std_logic_vector((lpm_width-1) downto 0);
    		result		: out	std_logic_vector((lpm_width-1) downto 0));
end Mbuf;

architecture archlpmstd of Mbuf is
begin

    B: for i in data'range generate
	U: buf port map (data(i), result(i));
    end generate;

end archlpmstd;

--
--	MFF
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mgates_local.all;

entity Mff is
    generic(lpm_width	: positive;
	    lpm_fftype	: fflop_type;
	    lpm_avalue	: string;
	    lpm_svalue	: string;
	    lpm_pvalue	: string;
	    lpm_hint	: goal_type);
    port(   data	: in	std_logic_vector((lpm_width-1) downto 0);
	    clock	: in	std_logic;
	    enable	: in	std_logic;
	    q		: out	std_logic_vector((lpm_width-1) downto 0);
	    aset	: in	std_logic;
	    aclr	: in	std_logic;
	    aload	: in	std_logic;
	    sset	: in	std_logic;
	    sclr	: in	std_logic;
	    sload	: in	std_logic;
	    testenab	: in	std_logic;
	    testin	: in	std_logic;
	    testout	: out	std_logic);
end Mff;

architecture archlpmstd of Mff is

    signal qtmp,d,t: std_logic_vector(q'range);
    signal async,set,clr : std_logic_vector(q'range);

begin

    async <= restring(lpm_avalue,lpm_width);

    -- Determine the value of the Input to the register
    process (data,sset,sclr,sload,enable,testin,testenab,qtmp)
    begin
	if (testenab = '1') then
	    d(0) <= testin;
	    if (lpm_width > 1) then
		d(q'high downto 1) <= qtmp((qtmp'high - 1) downto 0);
	    end if;
	elsif (sclr = '1' AND enable = '1') then
	    d <= (others => '0');
	elsif (sset = '1' AND enable = '1') then
	    d <= restring(lpm_svalue,lpm_width);
	elsif (sload = '1' AND enable = '1') then
	    d <= data;
	elsif (enable = '1') then
	    if (lpm_fftype = lpm_tff) then
		d <= data XOR qtmp;
	    else
		d <= data;
	    end if;
	else
	    d <= qtmp;
	end if;
    end process;

    G0: for j in q'range generate
	set(j) <= (aset AND     async(j)) OR (aload AND     data(j));
	clr(j) <= (aset AND NOT async(j)) OR (aload AND NOT data(j)) OR aclr;

	D0: if (lpm_fftype = lpm_dff) generate
	    U0: dsrff port map (d(j), set(j), clr(j), clock, qtmp(j));
	end generate;

	T0: if (lpm_fftype = lpm_tff) generate
	    t(j) <= d(j) XOR qtmp(j);
	    U0: tsrff port map (t(j), set(j), clr(j), clock, qtmp(j));
	end generate;
    end generate;

    testout <= qtmp(q'high);
    q <= qtmp;

end archlpmstd;

--
--	MBUSTRI
--

library ieee;
use ieee.std_logic_1164.all;

use work.Mgates_local.all;
entity Mbustri is
    generic(lpm_width	: positive;
	    lpm_hint	: goal_type);
    port(   tridata	: inout	std_logic_vector((lpm_width-1) downto 0);
	    data	: in	std_logic_vector((lpm_width-1) downto 0);
	    enabletr	: in	std_logic;
	    enabledt	: in	std_logic;
	    result	: out	std_logic_vector((lpm_width-1) downto 0));
end Mbustri;

architecture archlpmstd of Mbustri is

    signal yfb : std_logic_vector (result'range);

begin

    G0: for i in result'range generate
	D0: if (result(i)'port_state_att /= ps_open) generate
	    D00: if (port_state(enabletr)  = ps_tied_high) generate
		result(i) <= '1' when (yfb(i) = '1') else '0';
	    end generate;
	    D01: if (port_state(enabletr) /= ps_tied_high) generate
		U0: triout port map (yfb(i),enabletr,result(i));
	    end generate;
	end generate;
	D1: if ((tridata(i)'port_state_att /= ps_open) OR
		(result(i)'port_state_att /= ps_open)) generate
	    D10: if (port_state(enabledt)  = ps_tied_high) generate
		tridata(i) <= '1' when (data(i) = '1') else '0';
		yfb(i) <= data(i);
	    end generate;
	    D11: if (port_state(enabledt) /= ps_tied_high) generate
		U1: bufoe port map (data(i),enabledt,tridata(i),yfb(i));
	    end generate;
	end generate;
    end generate;

end archlpmstd;
