--------------------------------------------------------------------------------
--
--  Copyright Cypress Semiconductor Corporation, 1995,1996
--  as an unpublished work.
--
--------------------------------------------------------------------------------
--  Module generation package components
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

package mod_sh_local_ss is

    function string_to_natural (a : string) return natural;

end mod_sh_local_ss;

library ieee;
use ieee.std_logic_1164.all;

package body mod_sh_local_ss is

    function string_to_natural (a : string) return natural is
	variable result, indx : natural;
    begin
	result := 0;
	indx := 0;
	for i in a'reverse_range loop
	    if (a(i) = '1') then
		result := result + (2**indx);
	    end if;
	    indx := indx + 1;
	end loop;
	return result;
    end string_to_natural;

end mod_sh_local_ss;

--
--	sll_1s1c
--

library ieee;
use ieee.std_logic_1164.all;

use work.mod_sh_local_ss.all;

entity warp_sll_1s1c_ss is
    generic (c : string; goal : goal_type := speed);
    port (a : in std_logic_vector; r : out std_logic_vector);
end warp_sll_1s1c_ss;

architecture module of warp_sll_1s1c_ss is

    signal result : std_logic_vector ((r'length-1) downto 0);
    signal invar  : std_logic_vector ((a'length-1) downto 0);
    constant size : natural :=  string_to_natural(c);

begin

    invar <= a;
    G0: if (size >= invar'length) generate
	result <= (others => '0');
    end generate;

    G1: if (size <  invar'length) generate
	result(result'high downto size) <= invar((invar'high-size) downto 0);
	G2: if (size /= 0) generate
	    result((size-1) downto 0) <= (others => '0');
	end generate;
    end generate;

    r <= result;

end module;

--
--	srl_1s1c
--

library ieee;
use ieee.std_logic_1164.all;

use work.mod_sh_local_ss.all;

entity warp_srl_1s1c_ss is
    generic (c : string; goal : goal_type := speed);
    port (a : in std_logic_vector; r : out std_logic_vector);
end warp_srl_1s1c_ss;

architecture module of warp_srl_1s1c_ss is

    signal result : std_logic_vector(r'length-1 downto 0);
    signal invar  : std_logic_vector ((a'length-1) downto 0);
    constant size : natural :=  string_to_natural(c);

begin

    invar <= a;
    G0: if (size >= invar'length) generate
	result <= (others => '0');
    end generate;

    G1: if (size <  invar'length) generate
	G2: if (size /= 0) generate
	    result(result'high downto (result'high-size+1)) <= (others => '0');
	end generate;
	result((result'high-size) downto 0) <= invar(invar'high downto size);
    end generate;

    r <= result;

end module;

--
--	sla_1s1c
--

library ieee;
use ieee.std_logic_1164.all;

use work.mod_sh_local_ss.all;

entity warp_sla_1s1c_ss is
    generic (c : string; goal : goal_type := speed);
    port (a : in std_logic_vector; r : out std_logic_vector);
end warp_sla_1s1c_ss;

architecture module of warp_sla_1s1c_ss is

    signal result : std_logic_vector ((r'length-1) downto 0);
    signal invar  : std_logic_vector ((a'length-1) downto 0);
    constant size : natural :=  string_to_natural(c);

begin

    invar <= a;
    G0: if (size >= invar'length) generate
	result <= (others => invar(0));
    end generate;

    G1: if (size <  invar'length) generate
	result(result'high downto size) <= invar((invar'high-size) downto 0);
	G2: if (size /= 0) generate
	    result((size-1) downto 0) <= (others => invar(0));
	end generate;
    end generate;

    r <= result;

end module;

--
--	sra_1s1c
--

library ieee;
use ieee.std_logic_1164.all;

use work.mod_sh_local_ss.all;

entity warp_sra_1s1c_ss is
    generic (c : string; goal : goal_type := speed);
    port (a : in std_logic_vector; r : out std_logic_vector);
end warp_sra_1s1c_ss;

architecture module of warp_sra_1s1c_ss is

    signal result : std_logic_vector(r'length-1 downto 0);
    signal invar  : std_logic_vector ((a'length-1) downto 0);
    constant size : natural :=  string_to_natural(c);

begin

    invar <= a;
    G0: if (size >= invar'length) generate
	result <= (others => invar(invar'high));
    end generate;

    G1: if (size <  invar'length) generate
	G2: if (size /= 0) generate
	    result(result'high downto (result'high-size+1)) <=
				    (others => invar(invar'high));
	end generate;
	result((result'high-size) downto 0) <= invar(invar'high downto size);
    end generate;

    r <= result;

end module;

--
--	rol_1s1c
--

library ieee;
use ieee.std_logic_1164.all;

use work.mod_sh_local_ss.all;

entity warp_rol_1s1c_ss is
    generic (c : string; goal : goal_type := speed);
    port (a : in std_logic_vector; r : out std_logic_vector);
end warp_rol_1s1c_ss;

architecture module of warp_rol_1s1c_ss is

    signal result : std_logic_vector ((r'length-1) downto 0);
    signal invar  : std_logic_vector ((a'length-1) downto 0);
    constant size : natural :=  string_to_natural(c);

begin

    invar <= a;
    G0: if (size >= invar'length) generate
	result <= (others => '0');
    end generate;

    G1: if (size <  invar'length) generate
	result(result'high downto size) <= invar((invar'high-size) downto 0);
	G2: if (size /= 0) generate
	    result((size-1) downto 0) <=
				invar(invar'high downto (invar'high-size+1));
	end generate;
    end generate;

    r <= result;

end module;

--
--	ror_1s1c
--

library ieee;
use ieee.std_logic_1164.all;

use work.mod_sh_local_ss.all;

entity warp_ror_1s1c_ss is
    generic (c : string; goal : goal_type := speed);
    port (a : in std_logic_vector; r : out std_logic_vector);
end warp_ror_1s1c_ss;

architecture module of warp_ror_1s1c_ss is

    signal result : std_logic_vector(r'length-1 downto 0);
    signal invar  : std_logic_vector ((a'length-1) downto 0);
    constant size : natural :=  string_to_natural(c);

begin

    invar <= a;
    G0: if (size >= invar'length) generate
	result <= (others => '0');
    end generate;

    G1: if (size <  invar'length) generate
	G2: if (size /= 0) generate
	    result(result'high downto (result'high-size+1)) <=
				    invar(size-1 downto 0);
	end generate;
	result((result'high-size) downto 0) <= invar(invar'high downto size);
    end generate;

    r <= result;

end module;
