--      lpmpkg.vhp							      --
--									      --
--      Copyright Cypress Semiconductor Corporation, 1995,1996		      --
--        as an unpublished work.					      --
--									      --
--------------------------------------------------------------------------------
--			LPM Component Declarations			      --
--------------------------------------------------------------------------------

use work.cypress.x01z;
use work.cypress.x01z_vector;

package lpmpkg is

    type bvm		is array (integer range <>,integer range <>) of bit;
    type Shift_Type	is (LPM_LOGICAL, LPM_ROTATE, LPM_ARITHMETIC);
    type Repre_Type	is (LPM_SIGNED, LPM_UNSIGNED, LPM_GRAY);
    type Truth_Type	is (LPM_F, LPM_FD, LPM_FR, LPM_FDR);
    type CtDir_Type	is (LPM_NO_DIR, LPM_UP, LPM_DOWN);
    type Arith_Type	is (LPM_NO_TYP, LPM_ADD, LPM_SUB);
    type Fflop_Type	is (LPM_DFF,LPM_TFF);
    type ShDir_Type	is (LPM_LEFT,LPM_RIGHT);
    type goal_type	is (speed,area,combinatorial);
    type port_state_type is (ps_tied_low,ps_tied_high,ps_open,ps_connected);

-- These functions are defined only for simulation purposes.
    function port_state (arg : bit) return port_state_type;
    function connects (x : bit; i : integer) return integer;
    function max (a,b : natural) return natural;
    function vectorize (a, size : natural) RETURN bit_vector;
    function restring (a : string; w : positive) return bit_vector;

    component Mcnstnt
	generic(lpm_width	: positive;
		lpm_cvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(	result		: out	bit_vector(lpm_width - 1 downto 0));
    end component;
    component Minv
	generic(lpm_width	: positive;
		lpm_hint	: goal_type := speed);
	port(	data		: in	bit_vector(lpm_width - 1 downto 0);
		result		: out	bit_vector(lpm_width - 1 downto 0));
    end component;
    component Mand
	generic(lpm_width	: positive;
		lpm_size	: positive;
		lpm_hint	: goal_type := speed;
		lpm_data_pol    : string := "";
		lpm_result_pol  : string := "");
	port(	data		: in	bit_vector((lpm_width*lpm_size)-1
								    downto 0);
		result		: out	bit_vector(lpm_width - 1 downto 0));
    end component;
    component Mor
	generic(lpm_width	: positive;
		lpm_size	: positive;
		lpm_hint	: goal_type := speed;
		lpm_data_pol    : string := "";
		lpm_result_pol  : string := "");
	port(	data		: in	bit_vector((lpm_width*lpm_size)-1
								    downto 0);
		result		: out	bit_vector(lpm_width - 1 downto 0));
    end component;
    component Mxor
	generic(lpm_width	: positive;
		lpm_size	: positive;
		lpm_hint	: goal_type := speed;
		lpm_data_pol	: string := "";
		lpm_result_pol	: string := "");
	port(	data		: in	bit_vector((lpm_width*lpm_size)-1
								    downto 0);
		result		: out	bit_vector(lpm_width - 1 downto 0));
    end component;
    component Mbustri
	generic(lpm_width	: positive;
		lpm_hint	: goal_type := speed);
	port(	tridata		: inout	x01z_vector(lpm_width -1 downto 0);
		data		: in	bit_vector(lpm_width - 1 downto 0);
		enabletr	: in	bit;
		enabledt	: in	bit;
		result		: out	x01z_vector(lpm_width - 1 downto 0));
    end component;
    component Mmux
	generic(lpm_width	: positive;
		lpm_size	: positive;
		lpm_widths	: positive;
		lpm_hint	: goal_type := speed);
	port(	data		: in	bit_vector((lpm_width*lpm_size)-1
								 downto 0);
		sel		: in	bit_vector(lpm_widths -1 downto 0);
		result		: out	bit_vector(lpm_width - 1 downto 0));
    end component;
    component Mdecode
	generic(lpm_width	: positive;
		lpm_decodes	: positive;
		lpm_hint	: goal_type := speed);
	port(	data		: in	bit_vector(lpm_width - 1 downto 0);
		enable		: in	bit;
		eq		: out	bit_vector(lpm_decodes-1 downto 0));
    end component;
    component Mclshift
	generic(lpm_width	: positive;
		lpm_widthdist	: natural := 0;
		lpm_shifttype	: shift_type := lpm_logical;
		lpm_hint	: goal_type := speed);
	port(	data		: in	bit_vector(lpm_width - 1 downto 0);
		distance	: in	bit_vector(lpm_widthdist - 1 downto 0);
		direction 	: in	bit;
		result		: out	bit_vector(lpm_width - 1 downto 0);
		overflow	: out	bit;
		underflow	: out	bit);
    end component;
    component Madd_sub
	generic(lpm_width	: positive;
		lpm_representation : repre_type := lpm_unsigned;
		lpm_direction	: arith_type := lpm_no_typ;
		lpm_hint	: goal_type := speed);
	port(	dataA		: in	bit_vector(lpm_width - 1 downto 0);
		dataB		: in	bit_vector(lpm_width - 1 downto 0);
		cin		: in	bit;
		add_sub		: in	bit;
		result		: out	bit_vector(lpm_width - 1 downto 0);
		cout		: out	bit;
		overflow	: out   bit);
    end component;
    component Mcompare
	generic(lpm_width	: positive;
		lpm_representation : repre_type := lpm_unsigned;
		lpm_hint	: goal_type := speed);
	port(	dataA		: in	bit_vector(lpm_width - 1 downto 0);
		dataB		: in	bit_vector(lpm_width - 1 downto 0);
		alb		: out	bit;
		aeb		: out	bit;
		agb		: out	bit;
		ageb		: out   bit;
		aleb		: out   bit;
		aneb		: out   bit);
    end component;
    component Mmult
	generic(lpm_widthA	: positive;
		lpm_widthB	: positive;
		lpm_widthS	: natural := 0;
		lpm_widthP	: positive;
		lpm_representation : repre_type := lpm_unsigned;
		lpm_hint	: goal_type := speed);
	port(	dataA		: in	bit_vector(lpm_widthA - 1 downto 0);
		dataB		: in	bit_vector(lpm_widthB - 1 downto 0);
		sum		: in	bit_vector(lpm_widthS - 1 downto 0);
		result		: out	bit_vector(lpm_widthP - 1 downto 0));
    end component;
    component Mcounter
	generic(lpm_width	: positive;
		lpm_direction	: ctdir_type := lpm_no_dir;
		lpm_avalue	: string := "";
		lpm_svalue	: string := "";
		lpm_pvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(	data		: in	bit_vector(lpm_width - 1 downto 0);
		clock		: in	bit;
		clk_en		: in	bit;
		cnt_en		: in	bit;
		updown		: in	bit;
		q		: out	bit_vector(lpm_width - 1 downto 0);
		aset		: in	bit;
		aclr		: in	bit;
		aload		: in	bit;
		sset		: in	bit;
		sclr		: in	bit;
		sload		: in	bit;
		testenab	: in	bit;
		testin		: in	bit;
		testout		: out	bit);
    end component;
    component Mlatch
	generic(lpm_width	: positive;
		lpm_avalue	: string := "";
		lpm_pvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(	data		: in	bit_vector(lpm_width - 1 downto 0);
		gate		: in	bit;
		q		: out	bit_vector(lpm_width - 1 downto 0);
		aset		: in	bit;
		aclr		: in	bit);
    end component;
    component Mdff
	generic(lpm_width	: positive;
		lpm_avalue	: string := "";
		lpm_svalue	: string := "";
		lpm_pvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(	data		: in	bit_vector(lpm_width - 1 downto 0);
		clock		: in	bit;
		enable		: in	bit;
		shiftin		: in	bit;
		shiften		: in	bit;
		q		: out	bit_vector(lpm_width - 1 downto 0);
		shiftout	: out	bit;
		aset		: in	bit;
		aclr		: in	bit;
		sset		: in	bit;
		sclr		: in	bit;
		testenab	: in	bit;
		testin		: in	bit;
		testout		: out	bit);
    end component;
    component Mshiftreg
	generic(lpm_width	: positive;
		lpm_direction	: shdir_type:= lpm_left;
		lpm_avalue	: string := "";
		lpm_svalue	: string := "";
		lpm_pvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(   data		: in	bit_vector((lpm_width-1) downto 0);
		clock		: in	bit;
		enable		: in	bit;
		shiftin		: in	bit;
		load		: in	bit;
		q		: out	bit_vector((lpm_width-1) downto 0);
		shiftout	: out	bit;
		aset		: in	bit;
		aclr		: in	bit;
		sset		: in	bit;
		sclr		: in	bit;
		testenab	: in	bit;
		testin		: in	bit;
		testout		: out	bit);
    end component;
    component Mtff
	generic(lpm_width	: positive;
		lpm_avalue	: string := "";
		lpm_svalue	: string := "";
		lpm_pvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(	data		: in	bit_vector(lpm_width - 1 downto 0);
		clock		: in	bit;
		enable		: in	bit;
		q		: out	bit_vector(lpm_width - 1 downto 0);
		aset		: in	bit;
		aclr		: in	bit;
		aload		: in	bit;
		sset		: in	bit;
		sclr		: in	bit;
		sload		: in	bit;
		testenab	: in	bit;
		testin		: in	bit;
		testout		: out	bit);
    end component;
    component Mff
	generic(lpm_width	: positive;
		lpm_fftype	: fflop_type := lpm_dff;
		lpm_avalue	: string := "";
		lpm_svalue	: string := "";
		lpm_pvalue	: string := "";
		lpm_hint	: goal_type := speed);
	port(   data		: in	bit_vector((lpm_width-1) downto 0);
		clock		: in	bit;
		enable		: in	bit;
		q		: out	bit_vector((lpm_width-1) downto 0);
		aset		: in	bit;
		aclr		: in	bit;
		aload		: in	bit;
		sset		: in	bit;
		sclr		: in	bit;
		sload		: in	bit;
		testenab	: in	bit;
		testin		: in	bit;
		testout		: out	bit);
    end component;
    component MGND
	generic(lpm_width	: positive);
	port(	X		: out	bit_vector(lpm_width - 1 downto 0));
    end component;
    component MVCC
	generic(lpm_width	: positive);
	port(	X		: out	bit_vector(lpm_width - 1 downto 0));
    end component;
    component Mbuf
	generic(lpm_width	: positive;
		lpm_hint	: goal_type := speed);
	port(	data		: in	bit_vector(lpm_width - 1 downto 0);
		result		: out	bit_vector(lpm_width - 1 downto 0));
    end component;
end lpmpkg;

-- These functions are defined only for simulation purposes.
package body lpmpkg is
    function port_state (arg : bit) return port_state_type is
	variable result : port_state_type := ps_connected;
    begin

	return result;
    end port_state;

    function connects (x : bit; i : integer) return integer is
	variable result : integer := i;
    begin
	if (port_state(x) = ps_connected) then
	    result := result - 1;
	end if;
	return (result);
    end connects;

    function max (a,b : natural) return natural is
    begin
	if (a < b) then return b; else return a; end if;
    end max;

    function vectorize (a, size : natural) return bit_vector is
	variable result : bit_vector(size-1 downto 0) := (others => '0');
	variable b : natural;
	variable newsize : integer := size;
    begin
	if (newsize > 32) then 
	    newsize := 32;
	    assert (false)
	    report "VECTORIZE: Widths > 32 not allowed, vector truncated"
	    severity warning;
	end if;
	if (newsize <= 31) then
	    for i in result'reverse_range loop
		if ((a/(2**i)) mod 2) = 1 then
		    result(i) := '1';
		end if;
	    end loop;
	end if;
	if (newsize = 32) then
	    b := a/2;
	    if ((a mod 2) = 1) then
		result(result'low) := '1';
	    end if;
	    for i in result'low to (result'high - 1) loop
		if ((b/(2**i)) mod 2) = 1 then
		    result(i+1) := '1';
		end if;
	    end loop;
	end if;
	if newsize >= 31 then
	    if ((2**(newsize-2)) < (a/4)) then
		assert (false)
		report
		"VECTORIZE: Width too small to hold integer, vector truncated."
		severity warning;
	    end if;
	else
	    if ((2**newsize) < a) then
		assert (false)
		report
		"VECTORIZE: Width too small to hold integer, vector truncated."
		severity warning;
	    end if;
	end if;
	return result;
    end vectorize;
    function restring (a : string; w : positive) return bit_vector is
	variable result : bit_vector (w-1 downto 0) := (others => '1');
	variable indx : natural;
    begin

	if (a'length = 0) then
	    return result;
	elsif (a'length /= w) then
	    Assert (false)
	    Report "RESTRING: Source string length /= target vector width."
	    Severity Failure;
	    return result;
	else
	    indx := 0;
	    for i in a'reverse_range loop
		case a(i) is
		when '1' => result(indx) := '1';
		when '0' => result(indx) := '0';

		when others => Assert (false)

		    Report "Invalid char. literal (must be '1' or '0').  Defauting to  '0'."

		    Severity Warning;
		    result(indx) := '0';
		end case;
		indx := indx + 1;
	    end loop;
	    return result;
	end if;
    end restring;
end lpmpkg;

