--
--  Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--  as an unpublished work.
--

use work.cypress.all;

PACKAGE rtlpkg IS

attribute atomic_rtl of rtlpkg:PACKAGE is NOT_RTL_type;

component bufoe
    port(
	x	: in bit;		-- Logic input to Buffer.
	oe	: in bit;		-- Output Enable input.
	y	: inout x01z;		-- Output pin.
	yfb	: out bit		-- Output feed back as bit.
    );
end component;
component dltch
    port(
	d	: in bit;		-- D input to latch.
	e	: in bit;		-- Enabel, tranparent high.
	q	: out bit		-- Qutput from latch.
    );
end component;
component dff
    port(
	d	: in bit;		-- D input to register.
	clk	: in bit;		-- Positive edge Clock.
	q	: out bit		-- Output from register.
    );
end component;
component xdff
    port(
	x1	: in bit;		-- First Exclusive OR input.
	x2	: in bit;		-- Second Exclusive OR input.
	clk	: in bit;		-- Positive edge Clock.
	q	: out bit		-- Qutput from register.
    );
end component;
component jkff
    port(
	j	: in bit;		-- J input to J-K register.
	k	: in bit;		-- K input to J-K register.
	clk	: in bit;		-- Positive edge Clock.
	q	: out bit		-- Qutput from register.
    );
end component;
component buf
    port(
	x	: in bit;		-- Input to Buffer.
	y	: out bit		-- Output from Buffer.
    );
end component;
component srlch
    port(
	s	: in bit;		-- Set input, Async.
	r	: in bit;		-- Reset input, Async.
	q	: out bit		-- Output from Latch.
    );
end component;
component srff
    port(
	s	: in bit;		-- Set input, Sync.
	r	: in bit;		-- Reset inut, Sync.
	clk	: in bit;		-- Positive edge Clock.
	q	: out bit		-- Output from register.
    );
end component;
component dsrff
    port(
	d	: in bit;		-- D input to register.
	s	: in bit;		-- Set input Async.
	r	: in bit;		-- Reset input Async.
	clk	: in bit;		-- Positive edge Clock.
	q	: out bit		-- Output from register.
    );
end component;
component tsrff
    port(
	t	: in bit;		-- T input to register.
	s	: in bit;		-- Set input Async.
	r	: in bit;		-- Reset input Async.
	clk	: in bit;		-- Positive edge Clock.
	q	: out bit		-- Output from register.
    );
end component;
component tff
    port(
	t	: in bit;		-- Togal input to T type register.
	clk	: in bit;		-- Positive edge Clock.
	q	: out bit		-- Output from register.
    );
end component;
component xbuf                          -- Xor primitive gate
    port(
        x1      : in bit;               -- Xor input.
        x2      : in bit;               -- Xor input.
        q       : out bit               -- Xor output.
    );
end component;
component triout
    port(
	x	: in bit;		-- Logic input to Buffer.
	oe	: in bit;		-- Output Enable input.
	y	: out x01z		-- Output pin.
    );
end component;
component dsrlch 
    port (
	d	: in bit;
	s	: in bit;
	r	: in bit;
	e	: in bit;
	q	: out bit
    );
end component;

end rtlpkg;

--------------------------------------------------------------------------
--		Enties for Generic RTL.					--
--------------------------------------------------------------------------

USE WORK.rtlpkg.ALL;

use work.cypress.x01z;
use work.cypress.x01z_vector;

ENTITY bufoe IS
    PORT (
	x	: IN	bit;
	oe	: IN	bit;
	y	: INOUT	x01z;
	yfb	: OUT	bit);
END bufoe;

USE WORK.rtlpkg.ALL;
ENTITY dltch IS
    PORT (
	d	: IN bit;		-- D input to latch.
	e	: IN bit;		-- Enabel, tranparent high.
	q	: OUT bit		-- Qutput from latch.
    );
END dltch;

USE WORK.rtlpkg.ALL;
ENTITY dff IS
    PORT (
	d	: IN bit;		-- D input to register.
	clk	: IN bit;		-- Positive edge Clock.
	q	: OUT bit		-- Qutput from register.
    );
END dff;

USE WORK.rtlpkg.ALL;
ENTITY xdff IS
    PORT (
	x1	: IN bit;		-- First Exclusive OR input.
	x2	: IN bit;		-- Second Exclusive OR input.
	clk	: IN bit;		-- Positive edge Clock.
	q	: OUT bit		-- Qutput from register.
    );
END xdff;

USE WORK.rtlpkg.ALL;
ENTITY jkff IS
    PORT (
	j	: IN bit;		-- J input to J-K register.
	k	: IN bit;		-- K input to J-K register.
	clk	: IN bit;		-- Positive edge Clock.
	q	: OUT bit		-- Qutput from register.
    );
END jkff;

USE WORK.rtlpkg.ALL;
ENTITY buf IS
    PORT (
	x	: IN bit;
	y	: OUT bit);
END buf;

USE WORK.rtlpkg.ALL;
ENTITY srlch IS
    PORT (
	s	: IN bit;		-- Set input, Async.
	r	: IN bit;		-- Reset input, Async.
	q	: OUT bit		-- Output from Latch.
    );
END srlch;

USE WORK.rtlpkg.ALL;
ENTITY srff IS
    PORT (
	s	: IN bit;		-- Set input, Sync.
	r	: IN bit;		-- Reset inut, Sync.
	clk	: IN bit;		-- Positive edge Clock.
	q	: OUT bit		-- Output from register.
    );
END srff;

USE WORK.rtlpkg.ALL;
ENTITY dsrff IS	
    PORT (d	: IN bit;
	s	: IN bit;
	r	: IN bit;
	clk	: IN bit;
	q	: OUT bit);
END dsrff;

USE WORK.rtlpkg.ALL;
ENTITY tsrff IS	
    PORT (t	: IN bit;
	s	: IN bit;
	r	: IN bit;
	clk	: IN bit;
	q	: OUT bit);
END tsrff;

USE WORK.rtlpkg.ALL;
ENTITY tff IS
    PORT (
	t	: IN bit;		-- Togal input to T type register.
	clk	: IN bit;		-- Positive edge Clock.
	q	: OUT bit		-- Output from register.
    );
END tff;

USE WORK.rtlpkg.ALL;
ENTITY xbuf IS
    PORT (
	x1	: IN bit;		-- Xor input.
	x2	: IN bit;		-- Xor input.
	q	: OUT bit		-- Xor output.
    );
END xbuf;

USE WORK.rtlpkg.ALL;

use work.cypress.x01z;
use work.cypress.x01z_vector;

ENTITY triout IS
    PORT (
	x	: IN	bit;
	oe	: IN	bit;
	y	: OUT	x01z);
END triout;

USE WORK.rtlpkg.ALL;
ENTITY dsrlch IS
    PORT (
	D	: IN bit;
	S	: IN bit;
	R	: IN bit;
	E	: IN bit;
	Q	: OUT bit);
END dsrlch;

--------------------------------------------------------------------------
--		Architectures for Generic RTL				--
--------------------------------------------------------------------------

ARCHITECTURE archbufoe OF bufoe IS
BEGIN
    PROCESS (x, oe)
    BEGIN
	IF (oe = '1') THEN
	    if		(x = '0') then	y <= '0';
	    elsif	(x = '1') then	y <= '1';

	    else			y <= 'x';

	    end if;
	ELSE

	    y <= 'z';

	END IF;

    END PROCESS;

    PROCESS (y)
    BEGIN
	if (y = '0') then
	    yfb <= '0';
	else
	    yfb <= '1';
	end if;
    END PROCESS;
END archbufoe;

ARCHITECTURE archdltch OF dltch IS
    SIGNAL qx : bit;
BEGIN
    qx	<= (d and e) or (d and qx) or (not(e) and qx);
    q	<= qx;
END archdltch;

ARCHITECTURE archdff OF dff IS
BEGIN
      PROCESS
      BEGIN
	  WAIT UNTIL clk = '1';
	      q <= d;
      END PROCESS;
END archdff;

ARCHITECTURE archxdff OF xdff IS
    SIGNAL dx : bit;
BEGIN
    dx	<= x1 xor x2;
    u1 :dff port map(dx,clk,q);
END archxdff;

ARCHITECTURE archjkff OF jkff IS
    SIGNAL qx, dx : bit;
BEGIN
    dx	<= (j and not(qx)) or (not(k) and qx);
    q	<= qx;
    d1:dff port map(dx,clk,qx);
END archjkff;

ARCHITECTURE archbuf OF buf IS
BEGIN
    y	<= x;
END archbuf;
	
ARCHITECTURE archsrl OF srlch IS
    SIGNAL qx : bit;
BEGIN
    qx	<= (s and not(r)) or (qx and not(r));
    q	<= qx;
END archsrl;

ARCHITECTURE archsrff OF srff IS
    SIGNAL qx, dx : bit;
BEGIN
    dx  <= not ((not(s) or r) and (not (qx) or r));
    q	<= qx;
    d:dff port map(dx,clk,qx);
END archsrff;

ARCHITECTURE archdsrff OF dsrff IS
BEGIN
    PROCESS (r, s, clk)
    BEGIN
	IF (r = '1') THEN
	    q <= '0';
	ELSIF (s = '1') THEN
	    q <= '1';
	ELSIF (clk = '1' AND clk'event) THEN
	    q <= d;
	END IF;
    END PROCESS;
END archdsrff;

ARCHITECTURE archTSRFF of tsrff IS
   signal dtmp,qtmp : bit := '0';
BEGIN
    dtmp <= qtmp xor t;
    PROCESS (r, s, clk)
    BEGIN
	IF (r = '1') THEN
	    qtmp <= '0';
	ELSIF (s = '1') THEN
	    qtmp <= '1';
	ELSIF (clk = '1' AND clk'event) THEN
	    qtmp <= dtmp;
	END IF;
    END PROCESS;
    q <= qtmp;
END archTSRFF;

ARCHITECTURE archtff OF tff IS
    SIGNAL qx, dx : bit;
BEGIN
    dx	<= t xor qx;
    q	<= qx;
    d:dff port map(dx,clk,qx);
END archtff;

ARCHITECTURE archxbuf OF xbuf IS
BEGIN
    q	<= x1 xor x2;
END archxbuf;

ARCHITECTURE archtriout OF triout IS
BEGIN
    PROCESS (x, oe)
    BEGIN
	IF (oe = '1') THEN
	    if		(x = '0') then	y <= '0';
	    elsif	(x = '1') then	y <= '1';

	    else			y <= 'x';

	    end if;
	ELSE

	    y <= 'z';

	END IF;
    END PROCESS;
END archtriout;

ARCHITECTURE archdsrlch of dsrlch IS
    signal qi: bit;
BEGIN
    qi	<= (NOT(R) AND (S OR (D AND E) OR (qi AND NOT(E))));
    Q	<= qi;
end archdsrlch;
