--
--	Copyright Cypress Semiconductor Corporation, 1995,1996
--	  as an unpublished work.
--

library ieee;
use ieee.std_logic_1164.all;

PACKAGE iopkg IS

    COMPONENT CKPAD
	PORT ( 
	    p		: in std_logic;
	    q		: out std_logic);
    END COMPONENT;

    COMPONENT HDPAD
	PORT (
	    p		: in std_logic;
	    q		: out std_logic);
    END COMPONENT;

    COMPONENT HDIPAD
	PORT (
	    p		: in std_logic;
	    qn		: out std_logic);
    END COMPONENT;

    COMPONENT HD2PAD
	PORT (
	    p0,p1	: in std_logic;
	    q		: out std_logic);
    END COMPONENT;

    COMPONENT HD3PAD
	PORT (
	    p0,p1,p2	: in std_logic;
	    q		: out std_logic);
    END COMPONENT;

    COMPONENT HD4PAD
	PORT (
	    p0,p1,p2,p3	: in std_logic;
	    q		: out std_logic);
    END COMPONENT;

    COMPONENT INPAD
	PORT ( 
	    p		: inout std_logic;
	    q		: out std_logic);
    END COMPONENT;

    COMPONENT OUTPAD
	PORT ( 
	    a		: in std_logic;
	    p		: inout std_logic);
    END COMPONENT;

END iopkg;

------

library ieee;
use ieee.std_logic_1164.all;

ENTITY CKPAD IS
    PORT ( 
	p	: in std_logic;
	q	: out std_logic);
END CKPAD;

ARCHITECTURE archCKPAD OF CKPAD IS
BEGIN
    q <= p;
END archCKPAD;

------

library ieee;
use ieee.std_logic_1164.all;

ENTITY HDPAD IS
    PORT (
	p	: in std_logic;
	q	: out std_logic);
END HDPAD;

ARCHITECTURE archHDPAD OF HDPAD IS
BEGIN
    q <= p;
END archHDPAD;

------

library ieee;
use ieee.std_logic_1164.all;

ENTITY HDIPAD IS
    PORT (
	p	: in std_logic;
	qn	: out std_logic);
END HDIPAD;

ARCHITECTURE archHDIPAD OF HDIPAD IS
BEGIN
    qn <= not p;
END archHDIPAD;

------

library ieee;
use ieee.std_logic_1164.all;

ENTITY HD2PAD IS
    PORT (
	p0,p1	: in std_logic;
	q	: out std_logic);
END HD2PAD;

ARCHITECTURE archHD2PAD OF HD2PAD IS
BEGIN
    q <= p0 or p1;
END archHD2PAD;

------

library ieee;
use ieee.std_logic_1164.all;

ENTITY HD3PAD IS
    PORT (
	p0,p1,p2: in std_logic;
	q	: out std_logic);
END HD3PAD;

ARCHITECTURE archHD3PAD OF HD3PAD IS
BEGIN
    q <= p0 or p1 or p2;
END archHD3PAD;

------

library ieee;
use ieee.std_logic_1164.all;

ENTITY HD4PAD IS
    PORT (
	p0,p1,p2,p3 : in std_logic;
	q	: out std_logic);
END HD4PAD;

ARCHITECTURE archHD4PAD OF HD4PAD IS
BEGIN
    q <= p0 or p1 or p2 or p3;
END archHD4PAD;

------

library ieee;
use ieee.std_logic_1164.all;

ENTITY INPAD IS
    PORT (
	p	: inout std_logic;
	q	: out std_logic);
END INPAD;

ARCHITECTURE archINPAD OF INPAD IS
    signal zero : std_logic := '0';
component bufoe
    port(
	x	: in std_logic;		-- Logic input to Buffer.
	oe	: in std_logic;		-- Output Enable input.
	y	: inout std_logic;		-- Output pin.
	yfb	: out std_logic		-- Output feed back as std_logic.
    );
end component;
BEGIN
    U1: bufoe port map (zero,zero,p,q);
END archINPAD;

------

library ieee;
use ieee.std_logic_1164.all;

ENTITY OUTPAD IS
    PORT (
	a	: in std_logic;
	p	: inout std_logic);
END OUTPAD;

ARCHITECTURE archOUTPAD OF OUTPAD IS
    signal one : std_logic := '1';
component bufoe
    port(
	x	: in std_logic;		-- Logic input to Buffer.
	oe	: in std_logic;		-- Output Enable input.
	y	: inout std_logic;		-- Output pin.
	yfb	: out std_logic		-- Output feed back as std_logic.
    );
end component;
BEGIN
    U1: bufoe port map (a,one,p,open);
END archOUTPAD;

