--
--	Copyright Cypress Semiconductor Corporation, 1994,1995,1996
--	  as an unpublished work.
--

library ieee;
use ieee.std_logic_1164.all;

use work.c37xp.all;

ENTITY c37xoreg IS
    GENERIC (
	oregister	: oreg_type	:= dreg;
	source		: src_type	:= src_ptm;
	tCLK	:	time :=  4000 ps;
	tIRD	:	time :=  1000 ps;
	tICOMB	:	time :=  3000 ps;
	tILAT	:	time :=  5500 ps;
	tIRSU	:	time :=  6000 ps;
	tIRH	:	time :=  2000 ps;
	tSOP	:	time :=  1500 ps;
	tORD	:	time :=  1000 ps;
	tOCOMB	:	time :=  3000 ps;
	tOLAT	:	time :=  5500 ps;
	tORSU	:	time :=  4500 ps;
	tORH	:	time :=  0000 ps;
	tOD	:	time :=  1000 ps;
	tXZ	:	time :=  8500 ps;
	tZX	:	time :=  8500 ps;
	tFB	:	time :=  0000 ps;
	tRST	:	time :=  8500 ps;
	tPST	:	time :=  8500 ps;
	tWL	:	time :=  2500 ps;
	tWH	:	time :=  2500 ps;
	tRW	:	time :=  8000 ps;
	tRR	:	time := 10000 ps;
	tPW	:	time :=  8000 ps;
	tPR	:	time := 10000 ps);
    PORT (
	as	: in	std_logic;
	x	: in	std_logic;
	clk	: in	std_logic;
	ar	: in	std_logic;
	fb	: out	std_logic;
	q	: inout	std_logic := '0');

END c37xoreg;

ARCHITECTURE sim OF c37xoreg IS
    SIGNAL	regq_ts		: std_logic	:= '0';
    SIGNAL	regq_th		: std_logic	:= '0';
    SIGNAL	regq_twh	: std_logic	:= '0';
    SIGNAL	regq_twl	: std_logic	:= '0';
    SIGNAL	regq_tpw	: std_logic	:= '0';
    SIGNAL	regq_tpr	: std_logic	:= '0';
    SIGNAL	regq_trr	: std_logic	:= '0';
    SIGNAL	regq_trw	: std_logic	:= '0';
    SIGNAL	ireg_err	: std_logic	:= '0';
BEGIN

    -- check for clk setup/hold violations
    PROCESS (clk, x)

	VARIABLE xlastev, clklastev	: TIME := 0 ns;

    BEGIN

	-- only if not combinatorial
	if (oregister /= cmb) THEN
	    -- setup check
	    IF (clk'event AND (clk = '1')) THEN
		IF (source = src_ptm) THEN
		    ASSERT ((NOW <= Ticomb+Tsop) OR (NOW - xlastev) >= Torsu)
			REPORT
			"Setup ERROR ON X: Setting output to unknown" 
			SEVERITY WARNING;
		    IF NOT ((NOW <= Ticomb+Tsop)OR(NOW - xlastev) >= Torsu) THEN
			regq_ts <= '1'; 
			regq_ts <= transport '0' after 1 ns;
		    END IF;
		ELSE		-- Buried macrocell used as input register
		    ASSERT ((NOW = 0 ns) OR (NOW - xlastev) >= Tirsu)
			REPORT
			"Setup ERROR ON X: Setting output to unknown" 
			SEVERITY WARNING;
		    IF NOT ((NOW = 0 ns) OR (NOW - xlastev) >= Tirsu) THEN
			regq_ts <= '1'; 
			regq_ts <= transport '0' after 1 ns;
		    END IF;
		END IF;
		clklastev := NOW;
	    END IF;

	    -- hold check
	    IF (x'event) then
		IF (source = src_ptm) THEN
		    ASSERT ((NOW <= Ticomb+Tsop) OR ((NOW - clklastev) >= Torh))
			REPORT
			"Hold ERROR ON X: Setting output to unknown" 
			SEVERITY WARNING;
		    IF NOT((NOW <= Ticomb+Tsop)OR((NOW-clklastev) >= Torh)) THEN
			regq_th <= '1'; 
			regq_th <= transport '0' after 1 ns;
		    END IF;
		ELSE		-- Buried macrocell used as input register
		    ASSERT ((NOW = 0 ns) OR (NOW - clklastev) >= Tirh)
			REPORT
			"Hold ERROR ON X: Setting output to unknown" 
			SEVERITY WARNING;
		    IF NOT ((NOW = 0 ns) OR (NOW - clklastev) >= Tirh) THEN
			regq_ts <= '1'; 
			regq_ts <= transport '0' after 1 ns;
		    END IF;
		END IF;
		xlastev := NOW;
	    END IF;
	end if;
    END PROCESS;

    -- check clk pulse width
    PROCESS (clk)

	VARIABLE clklastevhi, clklastevlo : TIME := 0 ns;

    BEGIN

	-- only if not combinatorial.
	IF (oregister /= cmb) THEN
	    IF (clk = '0') THEN
		ASSERT ((NOW <= Ticomb+Tsop) OR ((NOW - clklastevhi) >= Twh))
		    REPORT
		    "Pulse Width (H) ERROR on CLK: Setting output to unknown"
		    SEVERITY WARNING;
		IF NOT ((NOW <= Ticomb+Tsop) OR ((NOW-clklastevhi) >= Twh)) THEN
		    regq_twh <= '1'; 
		    regq_twh <= transport '0' after 1 ns;
		END IF;
		clklastevlo := NOW;
	    ELSIF (clk = '1') THEN
	       ASSERT ((NOW <= Ticomb+Tsop) OR ((NOW - clklastevlo) >= Twl))
		    REPORT
		    "Pulse Width (L) ERROR on CLK: Setting output to unknown"
		    SEVERITY WARNING;
		IF NOT ((NOW <= Ticomb+Tsop) OR ((NOW-clklastevlo) >= Twl)) THEN
		    regq_twl <= '1'; 
		    regq_twl <= transport '0' after 1 ns;
		END IF;
		clklastevhi := NOW;
	    END IF;
	END IF;
    END PROCESS;

    -- check RESET minimum pulse width
    PROCESS (ar)

	VARIABLE arlastev : TIME := 0 ns;

    BEGIN
	-- only if not combinatorial
	if (oregister /= cmb) THEN
	    IF (ar = '0') THEN
		ASSERT ((NOW <= Ticomb+Tsop) OR ((NOW - arlastev) >= Trw))
		    REPORT
		    "Pulse Width ERROR on AR: Setting output to unknown"
		    SEVERITY WARNING;
		IF NOT ((NOW <= Ticomb+Tsop) OR ((NOW - arlastev) >= Trw)) THEN
		    regq_trw <= '1'; 
		    regq_trw <= transport '0' after 1 ns;
		END IF;
	    END IF;
	    arlastev := NOW;
	end if;
    END PROCESS;

    -- check PRESET minimum pulse width
    PROCESS (as)

	VARIABLE aslastev : TIME := 0 ns;

    BEGIN
	-- only if not combinatorial
	if (oregister /= cmb) THEN
	    IF (as = '0') THEN
		ASSERT ((NOW <= Ticomb+Tsop) OR ((NOW - aslastev) >= Tpw))
		    REPORT
		    "Pulse Width ERROR on AS: Setting output to unknown"
		    SEVERITY WARNING;
		IF NOT ((NOW <= Ticomb+Tsop) OR ((NOW - aslastev) >= Tpw)) THEN
		    regq_tpw <= '1'; 
		    regq_tpw <= transport '0' after 1 ns;
		END IF;
	    END IF;
	    aslastev := NOW;
	end if;
    END PROCESS;

    -- check for Asynchronous Reset Recovery violation
    PROCESS (clk, ar)

	VARIABLE arlastev, clklastev	: TIME := 0 ns;

    BEGIN
	-- only if not combinatorial
	if (oregister /= cmb) THEN
	    IF (ar'event) then
		arlastev := NOW;
	    END IF;
	    IF (ar = '0') THEN
		IF (clk'event AND (clk = '1')) THEN
		    ASSERT ((NOW <= Ticomb+Tsop) OR (NOW - arlastev) >= Trr)
			REPORT
			"Recovery ERROR on AR: Setting output to unknown" 
			SEVERITY WARNING;
		    IF NOT ((NOW <= Ticomb+Tsop) OR ((NOW - arlastev) >= Trr)) THEN
			regq_trr <= '1'; 
			regq_trr <= transport '0' after 1 ns;
		    END IF;
		    clklastev := NOW;
		END IF;
	    END IF;
	end if;
    END PROCESS;

    -- check for Asynchronous Preset Recovery violation
    PROCESS (clk, as)

	VARIABLE aslastev, clklastev	: TIME := 0 ns;

    BEGIN
	-- only if not combinatorial
	if (oregister /= cmb) THEN
	    IF (as'event) then
		aslastev := NOW;
	    END IF;
	    IF (as = '0') THEN
		IF (clk'event AND (clk = '1')) THEN
		    ASSERT ((NOW <= Ticomb+Tsop) OR (NOW - aslastev) >= Tpr)
			REPORT
			"Recovery ERROR on AS: Setting output to unknown" 
			SEVERITY WARNING;
		    IF NOT ((NOW <= Ticomb+Tsop) OR ((NOW - aslastev) >= Tpr)) THEN
			regq_tpr <= '1'; 
			regq_tpr <= transport '0' after 1 ns;
		    END IF;
		    clklastev := NOW;
		END IF;
	    END IF;
	end if;
    END PROCESS;

    -- Simulate the register.
    PROCESS (clk, x, as, ar, regq_ts, regq_th, regq_twl, regq_twh, regq_tpw,
   		 regq_tpr, regq_trr, regq_trw)

    BEGIN

	IF (source = src_ptm) THEN
	    IF (oregister = cmb) THEN
		q <= x AFTER Tocomb;
	    ELSIF ((regq_ts = '1') OR (regq_th  = '1') OR (regq_twh = '1')
		OR (regq_twl = '1') OR (regq_tpw = '1') OR (regq_tpr = '1')
		OR (regq_trr = '1') OR (regq_trw = '1')) THEN
		q <= 'X' AFTER Tord;
	    ELSIF (ar = '1') THEN
		q <= '0' AFTER Trst;
	    ELSIF (as = '1') THEN
		q <= '1' AFTER Tpst;
	    ELSIF (oregister = oltch) THEN
		IF (clk'event AND (clk = '1')) THEN
		    q <= x AFTER Tord;
		ELSIF (clk = '1') THEN	
		    q <= x AFTER Tolat;
		END IF;
	    ELSIF (oregister = dreg) THEN
		IF (clk'event AND (clk = '1')) THEN
		    q <= x AFTER Tord;
		END IF;
	    ELSIF (oregister = treg) THEN
		IF (clk'event AND (clk = '1')) THEN
		    IF (x = '1') THEN
			q <= not q AFTER Tord;
		    END IF;
		END IF;
	    END IF;
	ELSE			-- Buried macrocell used as input register.
	    ASSERT (oregister /= cmb)
		REPORT
		"Illegal configuration: Configured as input combinatorial"
		SEVERITY ERROR;
	    IF (oregister = cmb) THEN
		ireg_err <= '1'; 
		ireg_err <= transport '0' after 1 ns;
	    END IF;
	    ASSERT (oregister /= treg)
		REPORT
		"Illegal configuration: Configured as input T-register"
		SEVERITY ERROR;
	    IF (oregister = treg) THEN
		ireg_err <= '1'; 
		ireg_err <= transport '0' after 1 ns;
	    END IF;
	    IF ((regq_ts = '1') OR (regq_th  = '1') OR (regq_twh = '1')
		OR (regq_twl = '1') OR (regq_tpw = '1') OR (regq_tpr = '1')
		OR (regq_trr = '1') OR (regq_trw = '1') OR (ireg_err = '1'))THEN
		q <= 'X' AFTER Tird;
	    ELSIF (ar = '1') THEN
		q <= '0' AFTER Trst;
	    ELSIF (as = '1') THEN
		q <= '1' AFTER Tpst;
	    ELSIF (oregister = oltch) THEN
		IF (clk'event AND (clk = '1')) THEN
		    q <= x AFTER Tird;
		ELSIF (clk = '1') THEN	
		    q <= x AFTER Tilat;
		END IF;
	    ELSIF (oregister = dreg) THEN
		IF (clk'event AND (clk = '1')) THEN
		    q <= x AFTER Tird;
		END IF;
	    END IF;
	END IF;
    END PROCESS;

    -- Simulate the feedback.
    PROCESS (q)

    BEGIN

	IF (source = src_ptm) THEN
	    fb <= q AFTER Tfb;
	ELSE
	    fb <= q;
	END IF;

    END PROCESS;
END sim;
